using System;
using Microsoft.ContentManagement.Publishing;
using AndrewConnell.ContentManagement.Extensions;


namespace AndrewConnell.ContentManagement.Examples.CpDbEx
{
	class CodeSample
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			Console.WriteLine("CustomPropertyDbEx Sample Application");
			Console.WriteLine("");
			Console.WriteLine("");

			// establish context
			CmsApplicationContext cmsApp = new CmsApplicationContext();
			cmsApp.AuthenticateAsCurrentUser(PublishingMode.Published);
			Channel channel = (Channel)cmsApp.Searches.GetByPath("/Channels/Divisions/NorthAmerica");

			// create properties for this channel
			//CreateProperties(channel);

			// get the properties & display them
			Console.WriteLine(" -+-+-+-+-+-+-+-+-+-+-+-+-");
			Console.WriteLine("Display all properties for a specific channel using the following call:");
			Console.WriteLine("   CustomPropertyDbExGlobal.Searches.GetProperties(channel)");
			Console.WriteLine("");
			DisplayProperties(GetPropertiesByObject(channel));

			// get the properties & display them
			Console.WriteLine(" -+-+-+-+-+-+-+-+-+-+-+-+-");
			Console.WriteLine("Display specific property using the following call:");
			Console.WriteLine("   CustomPropertyDbExGlobal.Searches.GetProperties(channel, \"TimeZone\")");
			Console.WriteLine("");
			DisplayProperties(GetSpecificProperty(channel, "TimeZone"));

			// get the properties & display them
			Console.WriteLine(" -+-+-+-+-+-+-+-+-+-+-+-+-");
			Console.WriteLine("Display multi-property for a specific asset using the following call:");
			Console.WriteLine("   CustomPropertyDbExGlobal.Searches.GetProperties(channel, \"PointOfContact\")");
			Console.WriteLine("");
			DisplayProperties(GetSpecificProperty(channel, "PointOfContact"));

			Console.WriteLine("Press any key to continue...");
			Console.ReadLine();
		}



		static void CreateProperties(Channel channel)
		{
			CustomPropertyDbEx cp;

			// timezone
			cp = new CustomPropertyDbEx(channel);
			cp.PropertyKey = "TimeZone";
			cp.PropertyValue = "EST";
			cp.Commit();

			// two points of contact
			cp = new CustomPropertyDbEx(channel);
			cp.PropertyKey = "PointOfContact";
			cp.PropertyValue = "John Doe";
			cp.Commit();
			cp = new CustomPropertyDbEx(channel, "PointOfContact", "Jane Doe");
			cp.Commit();
		}



		static CustomPropertyDbExCollection GetPropertiesByObject(Channel channel)
		{
			return CustomPropertyDbExGlobal.Searches.GetProperties(channel);
		}


		static CustomPropertyDbExCollection GetSpecificProperty(Channel channel, string key)
		{
			return CustomPropertyDbExGlobal.Searches.GetProperties(channel, key);
		}


		static void DisplayProperties(CustomPropertyDbExCollection cpCollection)
		{
			foreach(CustomPropertyDbEx cp in cpCollection)
			{
				DisplayProperty(cp);
				Console.WriteLine(Environment.NewLine);
			}
		}


		static void DisplayProperty(CustomPropertyDbEx cp)
		{
			Console.WriteLine("Custom Property: {0}", cp.Guid.ToString());
			Console.WriteLine("  Asset Type: {0}", cp.AssetType.ToString());
			Console.WriteLine("  Asset GUID: {0}", cp.AssetGuid.ToString());
			Console.WriteLine("  Key:        {0}", cp.PropertyKey);
			Console.WriteLine("  Value:      {0}", cp.PropertyValue);
		}
	}
}