#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
/// Microsoft CMS 2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace AndrewConnell.ContentManagement.Extensions
{
	/// <summary>
	/// Summary description for CustomPropertyDbEx.
	/// </summary>
	public class CustomPropertyDbEx
	{
		#region Constants
		#endregion

		#region Objects
		// flag indicating this property has been marked for deletion
		private bool					m_deletePropertyFlag		=	false;
		// flag indicating this property exists in the database
		private bool					m_existingPropertyFlag	= false;

		private DataHandler		m_dataHandler;

		// properties of the custom prop
		private	Guid													m_uniqueID;
		private CustomPropertyDbExAssetType		m_assetType;
		private Guid													m_assetID;
		private string												m_cpKey				= string.Empty;
		private string												m_cpValue			= string.Empty;
		#endregion

		#region Constructor & Descructor
		/// <summary>
		/// Creates a new property for the specified channel.
		/// </summary>
		/// <param name="channel">MCMS channel.</param>
		public CustomPropertyDbEx(Channel channel)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Channel;
			this.m_assetID		= new Guid(channel.Guid);
		}

		
		/// <summary>
		/// Creates a new property for the specified channel.
		/// </summary>
		/// <param name="channel">MCMS channel.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		public CustomPropertyDbEx(Channel channel, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Channel;
			this.m_assetID		= new Guid(channel.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new property for the specified template.
		/// </summary>
		/// <param name="template">MCMS template.</param>
		public CustomPropertyDbEx(Template template)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Template;
			this.m_assetID		= new Guid(template.Guid);
		}

		
		/// <summary>
		/// Creates a new property for the specified template.
		/// </summary>
		/// <param name="template">MCMS template.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		public CustomPropertyDbEx(Template template, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Template;
			this.m_assetID		= new Guid(template.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new property for the specified resource.
		/// </summary>
		/// <param name="resource">MCMS resource.</param>
		public CustomPropertyDbEx(Resource resource)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Resource;
			this.m_assetID		= new Guid(resource.Guid);
		}

		
		/// <summary>
		/// Creates a new property for the specified resource.
		/// </summary>
		/// <param name="resource">MCMS resource.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		public CustomPropertyDbEx(Resource resource, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Resource;
			this.m_assetID		= new Guid(resource.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		
		/// <summary>
		/// Creates a new property for the specified posting.
		/// </summary>
		/// <param name="posting">MCMS posting.</param>
		public CustomPropertyDbEx(Posting posting)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Posting;
			this.m_assetID		= new Guid(posting.Guid);
		}


		/// <summary>
		/// Creates a new property for the specified posting.
		/// </summary>
		/// <param name="posting">MCMS posting.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		public CustomPropertyDbEx(Posting posting, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= CustomPropertyDbExAssetType.Posting;
			this.m_assetID		= new Guid(posting.Guid);
			this.m_cpKey		= propertyKey;
			this.m_cpValue		= propertyValue;
		}


		/// <summary>
		/// Create a new instance of a custom property.
		/// </summary>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		public CustomPropertyDbEx(CustomPropertyDbExAssetType assetType, Guid assetGuid)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();

			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
		}


		/// <summary>
		/// Create a new instance of a custom property.
		/// </summary>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		public CustomPropertyDbEx(CustomPropertyDbExAssetType assetType, Guid assetGuid, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();
			
			// init properties
			this.m_uniqueID		= Guid.NewGuid();
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
			this.m_cpKey			= propertyKey;
			this.m_cpValue		= propertyValue;
		}


		/// <summary>
		/// Create a new instance of a custom property... but only available to a loader or initalizer inside this class.
		/// </summary>
		/// <param name="propertyGuid">Unique ID of the property.</param>
		/// <param name="assetType">Type of the asset property is linked to.</param>
		/// <param name="assetGuid">Unique ID of the asset the the property is linked to.</param>
		/// <param name="propertyKey">Key of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		internal CustomPropertyDbEx(Guid propertyGuid, CustomPropertyDbExAssetType assetType, Guid assetGuid, string propertyKey, string propertyValue)
		{
			// init data handler
			this.m_dataHandler = new DataHandler();
			
			// set flags
			this.m_deletePropertyFlag			= false;
			this.m_existingPropertyFlag		= true;

			// set values
			this.m_uniqueID		= propertyGuid;
			this.m_assetType	= assetType;
			this.m_assetID		= assetGuid;
			this.m_cpKey			= propertyKey;
			this.m_cpValue		= propertyValue;
		}

		#endregion

		#region Private/protected/internal methods
		#endregion

		#region Public methods
		/// <summary>
		/// Clones the current custom property to a new custom property for a specific asset type and guid.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include Commit.</b>  You must call .Commit to save changes to the database.
		/// </remarks>
		/// <param name="newAssetType">Type of asset for new property.</param>
		/// <param name="newAssetGuid">Unique ID of asset for new property.</param>
		/// <returns>Copy of the current property, but with a specific asset type and asset id changed.</returns>
		public CustomPropertyDbEx Clone(CustomPropertyDbExAssetType newAssetType, Guid newAssetGuid)
		{
			return new CustomPropertyDbEx(this.Guid, newAssetType, newAssetGuid, this.PropertyKey, this.PropertyValue);
		}


		/// <summary>
		/// Ensures the property has the latest information from the database.
		/// </summary>
		/// <remarks>
		/// If any changes have been made before a commit, these changes will be lost.
		/// </remarks>
		public void Sync()
		{
			// make sure there's an object
			if (this.m_uniqueID.ToString().Length == 0)
				throw new Exception("No custom property currently loaded to sync.");


			// get property
			DataRow dr = this.m_dataHandler.Select_ByID(this.m_uniqueID);


			// refresh property values
			switch (dr[CustomPropertyTableField.ID.ToString()].ToString())
			{
				case "Site":
					this.m_assetType = CustomPropertyDbExAssetType.Site;
					break;
				case "Channel":
					this.m_assetType = CustomPropertyDbExAssetType.Channel;
					break;
				case "Template":
					this.m_assetType = CustomPropertyDbExAssetType.Template;
					break;
				case "Resource":
					this.m_assetType = CustomPropertyDbExAssetType.Resource;
					break;
				case "Posting":
					this.m_assetType = CustomPropertyDbExAssetType.Posting;
					break;
				case "Other":
					this.m_assetType = CustomPropertyDbExAssetType.Other;
					break;
			}
			this.m_assetID		= new Guid(dr[CustomPropertyTableField.AssetID.ToString()].ToString());
			this.m_cpKey			= dr[CustomPropertyTableField.PropertyKey.ToString()].ToString();
			this.m_cpValue		= dr[CustomPropertyTableField.PropertyValue.ToString()].ToString();


			// update flags
			this.m_existingPropertyFlag = false;
		}


		/// <summary>
		/// Saves all current settings of the current property to the database.
		/// </summary>
		public void Commit()
		{
			// make sure there's an object
			if (this.m_uniqueID.ToString().Length == 0)
				throw new Exception("No custom property currently loaded to commit.");


			// if delete..
			if (this.m_deletePropertyFlag)
			{
				#region delete current property
				this.m_dataHandler.DeleteProperty(this.m_uniqueID);

				// reset all internal objects
				// TODO: need to null-out the entire object
				this.m_existingPropertyFlag = false;
				this.m_deletePropertyFlag = false;
				this.m_cpKey = null;
				this.m_cpValue = null;
				#endregion
			}
				// else, if it already existed, update it...
			else if (this.m_existingPropertyFlag)
			{
				#region update existing current property
				this.m_dataHandler.Update(this.m_uniqueID, this.m_assetType.ToString(), this.m_assetID, this.m_cpKey, this.m_cpValue);
				#endregion
			}
				// else, create new one
			else
			{
				#region create new custom property
				this.m_dataHandler.Insert(this.m_uniqueID, this.m_assetType.ToString(), this.m_assetID, this.m_cpKey, this.m_cpValue);

				this.m_existingPropertyFlag = true;
				#endregion
			}
		}


		/// <summary>
		/// Removes the current property from the system.
		/// </summary>
		/// <remarks>
		/// Must call Commit() to take effect database.
		/// </remarks>
		public void Delete()
		{
			this.m_deletePropertyFlag = true;
		}
		#endregion

		#region Public properties
		/// <summary>
		/// Unique identifier of the.
		/// </summary>
		public Guid Guid
		{
			get {return this.m_uniqueID;}
		}


		/// <summary>
		/// Type of asset/object the custom property is assigned to.
		/// </summary>
		public CustomPropertyDbExAssetType AssetType
		{
			get{return this.m_assetType;}
			set{this.m_assetType = value;}
		}


		/// <summary>
		/// Unique ID of the 
		/// </summary>
		public Guid AssetGuid
		{
			get{return this.m_assetID;}
			set{this.m_assetID = value;}
		}


		/// <summary>
		/// Key of the property
		/// </summary>
		public string PropertyKey
		{
			get{return this.m_cpKey;}
			set{this.m_cpKey = value;}
		}


		/// <summary>
		/// Value of the property
		/// </summary>
		public string PropertyValue
		{
			get{return this.m_cpValue;}
			set{this.m_cpValue = value;}
		}
		#endregion

		#region Event Handlers
		#endregion

		#region Comparison objects
		#endregion	
	}
}