#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.Xml;
#endregion

namespace AndrewConnell.ContentManagement.Extensions
{
	/// <summary>
	/// Summary description for CustomPropertyDbExCollection.
	/// </summary>
	[Serializable]
	public class CustomPropertyDbExCollection : CollectionBase
	{
		#region Constructor
		/// <summary>
		/// Inits a new instance of the CustomPropertyDbExCollection class
		/// </summary>
		public CustomPropertyDbExCollection(){}

		
		/// <summary>
		/// Inits a new instance of the CustomPropertyDbExCollection class
		/// </summary>
		public CustomPropertyDbExCollection(CustomPropertyDbExCollection value)
		{
			this.AddRange(value);
		}

		
		/// <summary>
		/// Inits a new instance of the CustomPropertyDbExCollection class
		/// </summary>
		public CustomPropertyDbExCollection(CustomPropertyDbEx[] value)
		{
			this.AddRange(value);
		}
		#endregion

		#region Public class methods
		/// <summary>
		/// Adds the CustomPropertyDbEx object to this collections
		/// </summary>
		/// <param name="value"></param>
		/// <returns></returns>
		public int Add(CustomPropertyDbEx value)
		{
			return this.List.Add(value);
		}


		/// <summary>
		/// Copies the elements of the specified CustomPropertyDbEx array to the end of the collection.
		/// </summary>
		/// <param name="value">An array of type CustomPropertyDbEx containing the Components to add to the collection.</param>
		public void AddRange(CustomPropertyDbEx[] value) 
		{
			for (int i = 0;	(i < value.Length); i = (i + 1)) 
			{
				this.Add(value[i]);
			}
		}


		/// <summary>
		/// Adds the contents of another CustomPropertyDbExCollection to the end of the collection.
		/// </summary>
		/// <param name="value">A CustomPropertyDbExCollection containing the Components to add to the collection. </param>
		public void AddRange(CustomPropertyDbExCollection value) 
		{
			for (int i = 0;	(i < value.Count); i = (i +	1))	
			{
				this.Add((CustomPropertyDbEx)value.List[i]);
			}
		}
		

		/// <summary>
		/// Gets a value indicating whether the collection contains the specified CustomPropertyDbExCollection.
		/// </summary>
		/// <param name="value">The CustomPropertyDbExCollection to search for in the collection.</param>
		/// <returns><b>true</b> if the collection contains the specified object; otherwise, <b>false</b>.</returns>
		public bool Contains(CustomPropertyDbEx value) 
		{
			return this.List.Contains(value);
		}
		
		
		/// <summary>
		/// Copies the collection Components to a one-dimensional <see cref="T:System.Array">Array</see> instance beginning at the specified index.
		/// </summary>
		/// <param name="array">The one-dimensional <see cref="T:System.Array">Array</see> that is the destination of the values copied from the collection.</param>
		/// <param name="index">The index of the array at which to begin inserting.</param>
//		public void CopyTo(CustomPropertyDbEx[] array, int index) 
//		{
//			this.List.CopyTo(array, index);
//		}


		/// <summary>
		/// Gets the index in the collection of the specified CustomPropertyDbExCollection, if it exists in the collection.
		/// </summary>
		/// <param name="value">The CustomPropertyDbExCollection to locate in the collection.</param>
		/// <returns>The index in the collection of the specified object, if found; otherwise, -1.</returns>
		public int IndexOf(CustomPropertyDbEx value) 
		{
			return this.List.IndexOf(value);
		}
		

		/// <summary>
		/// Inserts the specified postings into the specified index
		/// </summary>
		/// <param name="index">index where the new element should be added</param>
		/// <param name="value">element to insert</param>
		public void Insert(int index, CustomPropertyDbEx value)	
		{
			List.Insert(index, value);
		}
	
		
		/// <summary>
		/// Removes the specified element from the collection
		/// </summary>
		/// <param name="value">element to remove from the collection</param>
		public void Remove(CustomPropertyDbEx value) 
		{
			List.Remove(value);
		}
		

		/// <summary>
		/// Returns an enumerator that can iterate through the CustomPropertyDbExCollection instance.
		/// </summary>
		/// <returns>An CustomPropertyDbExCollection for the CustomPropertyDbExCollection instance.</returns>
		public new CustomPropertyDbExCollectionEnumerator GetEnumerator()	
		{
			return new CustomPropertyDbExCollectionEnumerator(this);
		}
		
		
		/// <summary>
		/// Commits all items in the collection to the database.
		/// </summary>
		public void Commit()
		{
			// call Commit on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Commit();
		}


		/// <summary>
		/// Sync all items in the collection with the database (so they have the same contents as what's in the database.
		/// </summary>
		public void Sync()
		{
			// call Sync on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Sync();
		}


		/// <summary>
		/// Removes all items in the collection.
		/// </summary>
		/// <remarks>
		/// Must call Commit() to take effect in the database.
		/// </remarks>
		public void Delete()
		{
			// call Delete on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Delete();
		}


		/// <summary>
		/// Clones the current collection of custom properties to a new collection for a specific asset type and guid.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include Commit.</b>  You must call .CommitAll on the collection or .Commit on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="newAssetType">Type of asset for new collection.</param>
		/// <param name="newAssetGuid">Unique ID of asset for new collection.</param>
		/// <returns>Copy of the current collection, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(CustomPropertyDbExAssetType newAssetType, Guid newAssetGuid)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(newAssetType, newAssetGuid));

			return collection;
		}


		/// <summary>
		/// Sorts the collection according to the asset type, asset guid, then key, then value
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void Sort(SortDirection direction)
		{
			//TODO: implement this
		}


		/// <summary>
		/// Sorts the collection according to the then key
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void SortByKey(SortDirection direction)
		{
			//TODO: implement this
		}


		/// <summary>
		/// Sorts the collection according to the value
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void SortByValue(SortDirection direction)
		{
			//TODO: implement this
		}
		#endregion

		#region Enumeration construct
		/// <summary>
		/// Supports a simple iteration over a CustomPropertyDbExCollection.
		/// </summary>
		public class CustomPropertyDbExCollectionEnumerator : IEnumerator	
		{
			private	IEnumerator _enumerator;
			private	IEnumerable _temp;
			
			/// <summary>
			/// Initializes a new instance of the CustomPropertyDbExCollectionEnumerator class referencing the specified object.
			/// </summary>
			/// <param name="mappings">The CustomPropertyDbExCollection to enumerate.</param>
			public CustomPropertyDbExCollectionEnumerator(CustomPropertyDbExCollection mappings)
			{
				_temp =	((IEnumerable)(mappings));
				_enumerator = _temp.GetEnumerator();
			}
			
			/// <summary>
			/// Gets the current element in the collection.
			/// </summary>
			public CustomPropertyDbEx Current
			{
				get {return ((CustomPropertyDbEx)(_enumerator.Current));}
			}
			
			object IEnumerator.Current
			{
				get {return _enumerator.Current;}
			}
			
			/// <summary>
			/// Advances the enumerator to the next element of the collection.
			/// </summary>
			/// <returns><b>true</b> if the enumerator was successfully advanced to the next element; <b>false</b> if the enumerator has passed the end of the collection.</returns>
			public bool MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			/// <summary>
			/// Sets the enumerator to its initial position, which is before the first element in the collection.
			/// </summary>
			public void Reset()
			{
				_enumerator.Reset();
			}
			
			void IEnumerator.Reset() 
			{
				_enumerator.Reset();
			}
		}
		#endregion

		#region Public class properties
		/// <summary>
		/// Gets the CustomPropertyDbExCollection at the specified index in the collection.
		/// <para>
		/// In C#, this property is the indexer for the CustomPropertyDbExCollection class.
		/// </para>
		/// </summary>
		public CustomPropertyDbEx this[int index] 
		{
			get	{return ((CustomPropertyDbEx)(this.List[index]));}
		}
		#endregion
	}
}