#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
/// Microsoft CMS 2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace AndrewConnell.ContentManagement.Extensions
{
	/// <summary>
	/// Summary description for CustomPropertyDbExGlobal.
	/// </summary>
	public class CustomPropertyDbExGlobal
	{
		#region Constants
		#endregion

		#region Objects
		#endregion

		#region Constructor & Descructor
		/// <summary>
		/// Class constructor
		/// </summary>
		private CustomPropertyDbExGlobal(){}
		#endregion

		#region Private/protected/internal methods
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		#endregion

		#region SUBCLASS Searches
		public class Searches
		{
			#region Constants
			#endregion

			#region Objects
			private static DataHandler		m_dataHandler;
			#endregion

			#region Constructor & Descructor
			/// <summary>
			/// Class constructor
			/// </summary>
			private Searches(){}
			#endregion

			#region Private/protected/internal methods
			/// <summary>
			/// Loads the datahandler if it's not already initalized.
			/// </summary>
			/// <returns><see cref="DataHandlers" /></returns>
			private static DataHandler GetDataHandler()
			{
				if (m_dataHandler == null)
					m_dataHandler = new DataHandler();

				return m_dataHandler;
			}
			

			/// <summary>
			/// Finds the custom property(s) matching the specific asset guid.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetGuid"><see cref="System.Guid"/> unique ID of the desired asset.</param>
			/// <returns>Collection of <see cref="CustomPropertyDbEx"/> matching the found property.</returns>
			internal static CustomPropertyDbExCollection GetByAssetGuid(Guid assetGuid)
			{
				DataSet ds = GetDataHandler().Select_ByAssetID(assetGuid);

				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}
			

			/// <summary>
			/// Finds the custom property(s) matching the specific asset guid & key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetGuid"><see cref="System.Guid"/> unique ID of the desired asset.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns>Collection of <see cref="CustomPropertyDbEx"/> matching the found property.</returns>
			internal static CustomPropertyDbExCollection GetByKeyForAsset(Guid assetGuid, string key)
			{
				DataSet ds = GetDataHandler().Select_ByAssetID(assetGuid);

				// if properties found...
				if (ds != null)
				{
					// get all properties with specific key for this asset
					DataRow[] dr = ds.Tables[0].Select(
						string.Format("{0}='{1}'", CustomPropertyTableField.PropertyKey.ToString(), key), 
						string.Format("{0} ASC", CustomPropertyTableField.PropertyValue.ToString()), 
						DataViewRowState.OriginalRows);
				
					// if properties with this key found...
					if (dr.Length == 0)
						return null;
					else
					{
						CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();
						foreach(DataRow r in dr)
							collection.Add(Transform.ToCustomPropertyDbEx(r));

						return collection;
					}
				}
				else
					return null;
			}
			#endregion

			#region Public methods
			/// <summary>
			/// Retrives all custom properties for the specified channel.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="channel">MCMS Channel object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the channel.</returns>
			public static CustomPropertyDbExCollection GetProperties(Channel channel)
			{
				return GetByAssetGuid(new Guid(channel.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified channel & property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="channel">MCMS Channel object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the channel.</returns>
			public static CustomPropertyDbExCollection GetProperties(Channel channel, string key)
			{
				return GetByKeyForAsset(new Guid(channel.Guid.ToString()), key);
			}


			/// <summary>
			/// Retrives all custom properties for the specified template.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="template">MCMS Template object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the template.</returns>
			public static CustomPropertyDbExCollection GetProperties(Template template)
			{
				return GetByAssetGuid(new Guid(template.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified template & property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="template">MCMS Template object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the template.</returns>
			public static CustomPropertyDbExCollection GetProperties(Template template, string key)
			{
				return GetByKeyForAsset(new Guid(template.Guid.ToString()), key);
			}


			/// <summary>
			/// Retrives all custom properties for the specified posting.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="posting">MCMS Posting object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the posting.</returns>
			public static CustomPropertyDbExCollection GetProperties(Posting posting)
			{
				return GetByAssetGuid(new Guid(posting.Guid.ToString()));
			}
	

			/// <summary>
			/// Retrives all custom properties for the specified posting & property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="posting">MCMS Posting object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the posting.</returns>
			public static CustomPropertyDbExCollection GetProperties(Posting posting, string key)
			{
				return GetByKeyForAsset(new Guid(posting.Guid.ToString()), key);
			}
	

			/// <summary>
			/// Retrives all custom properties for the specified resource.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="posting">MCMS Resource object containing custom properties.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the resource.</returns>
			public static CustomPropertyDbExCollection GetProperties(Resource resource)
			{
				return GetByAssetGuid(new Guid(resource.Guid.ToString()));
			}


			/// <summary>
			/// Retrives all custom properties for the specified resource & property key.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="resource">MCMS Resource object containing custom properties.</param>
			/// <param name="key">Property key of property desired.</param>
			/// <returns><see cref="CustomPropertyDbExCollection" /> of all custom properties for the resource.</returns>
			public static CustomPropertyDbExCollection GetProperties(Resource resource, string key)
			{
				return GetByKeyForAsset(new Guid(resource.Guid.ToString()), key);
			}


			/// <summary>
			/// Finds the custom property matching a specific guid.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="propertyGuid"><see cref="System.Guid"/> unique ID of the desired property.</param>
			/// <returns><see cref="CustomPropertyDbEx"/> matching the found property.</returns>
			public static CustomPropertyDbEx GetByGuid(Guid propertyGuid)
			{
				DataRow r = GetDataHandler().Select_ByID(propertyGuid);
				if (r != null)
					return Transform.ToCustomPropertyDbEx(r);
				else
					return null;
			}


			/// <summary>
			/// Finds the custom property(s) matching a specific asset type.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="assetType"><see cref="CustomPropertyDbExAssetType" /> of the desired property(s).</param>
			/// <returns>Collection of <see cref="CustomPropertyDbEx"/> matching the found property.</returns>
			public static CustomPropertyDbExCollection GetByAssetType(CustomPropertyDbExAssetType assetType)
			{
				DataSet ds = GetDataHandler().Select(assetType.ToString(), null, null);
				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}



			/// <summary>
			/// Finds the custom property(s) matching the specific property value.
			/// </summary>
			/// <remarks>
			/// If no matching property found, null value returned (<b>nothing</b> in VB.NET).
			/// </remarks>
			/// <param name="value">Value of the desired property(s).</param>
			/// <returns>Collection of <see cref="CustomPropertyDbEx"/> matching the found property.</returns>
			public static CustomPropertyDbExCollection GetByValue(string value)
			{
				DataSet ds = GetDataHandler().Select_ByValue(value);

				if (ds != null)
					return Transform.ToCustomPropertyDbEx(ds);
				else
					return null;
			}
			#endregion
		}
		#endregion
	}
}