#region References
	using System;
	using System.Data;
	using System.Data.SqlClient;
#endregion

namespace AndrewConnell.ContentManagement.Extensions
	{
	/// <summary>
	/// Summary description for SqlUtility.
	/// </summary>
	internal class SqlUtility
	{
		#region Boilerplate
		#region Constructor & destructor
		/// <summary>
		/// SqlUtility constructor
		/// </summary>
		private SqlUtility() {}
		#endregion
		#endregion

		#region Exposed Class Methods
		/// <summary>
		/// Determines if the provided string is empty or not.
		/// </summary>
		/// <param name="text">string to test</param>
		/// <returns>System.DBNull if the string is empty... the string otherwise</returns>
		internal static object CheckForNullString(string text)
		{
			if(text == null || text.Trim().Length == 0)
				return System.DBNull.Value;
			else
				return text;
		}


		/// <summary>
		/// Creates a new SQL Command parameter
		/// </summary>
		/// <param name="paramName">name of the parameter</param>
		/// <param name="value">value of parameter</param>
		/// <returns></returns>
		internal static SqlParameter MakeInParam(string paramName, object value)
		{
			return new SqlParameter(paramName, value);
		}


		/// <summary>
		/// Creates a new input SQL Command parameter
		/// </summary>
		/// <param name="paramName">Name of param.</param>
		/// <param name="dbType">Param type.</param>
		/// <param name="size">Param size.</param>
		/// <param name="value">Param value.</param>
		/// <returns>New parameter.</returns>
		internal static SqlParameter MakeInParam(string paramName, SqlDbType dbType, int size, object value) 
		{
			return MakeParam(paramName, dbType, size, ParameterDirection.Input, value);
		}		


		/// <summary>
		/// Creates a new input SQL Command parameter
		/// </summary>
		/// <param name="paramName">Name of param.</param>
		/// <param name="dbType">Param type.</param>
		/// <param name="size">Param size.</param>
		/// <returns>New parameter.</returns>
		internal static SqlParameter MakeOutParam(string paramName, SqlDbType dbType, int size) 
		{
			return MakeParam(paramName, dbType, size, ParameterDirection.Output, null);
		}		


		/// <summary>
		/// Make stored procedure param.
		/// </summary>
		/// <param name="paramName">Name of param.</param>
		/// <param name="dbType">Param type.</param>
		/// <param name="size">Param size.</param>
		/// <param name="direction">Parm direction.</param>
		/// <param name="value">Param value.</param>
		/// <returns>New parameter.</returns>
		internal static SqlParameter MakeParam(string paramName, SqlDbType dbType, Int32 size, ParameterDirection direction, object value) 
		{
			SqlParameter param;

			if(size > 0)
				param = new SqlParameter(paramName, dbType, size);
			else
				param = new SqlParameter(paramName, dbType);

			param.Direction = direction;
			if (!(direction == ParameterDirection.Output && value == null))
				param.Value = value;

			return param;
		}


		/// <summary>
		/// Utility function to sort and filter a provided dataset.
		/// </summary>
		/// <param name="origDS">dataset to perform sort/filter on</param>
		/// <param name="sortCriteria">criteria to sort by (field [blank|ASC|DESC][, [n]])</param>
		/// <param name="filterCriteria">criteria to filter by (field =|<>|etc value[, [n]]></param>
		/// <remarks>Only sorts/filters the first table in the dataset.</remarks>
		/// <returns>DataSet filtered and sorted with provided criteria</returns>
		internal static DataSet SortAndFilterDataSet(DataSet origDS, string sortCriteria, string filterCriteria)
		{
			// convert to dataview
			DataView dv = new DataView(origDS.Tables[0]);

			// sort
			if (sortCriteria == string.Empty)
				dv.Sort =sortCriteria;

			//filter
			if (filterCriteria == string.Empty)
				dv.RowFilter = filterCriteria;

			// convert dataview to datatable
			DataTable dt = dv.Table.Clone();
			foreach (DataRowView drv in dv)
				dt.ImportRow(drv.Row);

			// add new datatable back to dataset
			DataSet newDS = new DataSet();
			newDS.Tables.Add(dt);

			//return new sorted/filtered dataset
			return newDS;
		}
		#endregion
	}
}