#region Namespace references
using System;
using System.Data;
#endregion

namespace AndrewConnell.ContentManagement.Extensions
{
	/// <summary>
	/// Transforms DataRows & DataSets to CustomPropertyDbEx objects & collections.
	/// </summary>
	internal class Transform
	{
		#region Constructor & Descructor
		private Transform(){}
		#endregion

		/// <summary>
		/// Convert a datarow to a <see cref="CustomPropertyDbEx" />.
		/// </summary>
		/// <param name="r"><see cref="DataRow" /></param>
		/// <returns><see cref="CustomPropertyDbEx" /></returns>
		internal static CustomPropertyDbEx ToCustomPropertyDbEx(DataRow r)
		{
			Guid id;
			CustomPropertyDbExAssetType cpType = CustomPropertyDbExAssetType.Other;
			Guid assetID;


			// make sure a row was actually passed
			if (r == null)
				throw new Exception("Null DataRow supplied.  You must pass a valid DataRow.");
			
			// get data from datarow
			id = new Guid(r[CustomPropertyTableField.ID.ToString()].ToString());
			assetID = new Guid(r[CustomPropertyTableField.AssetID.ToString()].ToString());
			switch (r[CustomPropertyTableField.AssetType.ToString()].ToString())
			{
				case "Site":
					cpType = CustomPropertyDbExAssetType.Site;
					break;
				case "Channel":
					cpType = CustomPropertyDbExAssetType.Channel;
					break;
				case "Template":
					cpType = CustomPropertyDbExAssetType.Template;
					break;
				case "Resource":
					cpType = CustomPropertyDbExAssetType.Resource;
					break;
				case "Posting":
					cpType = CustomPropertyDbExAssetType.Posting;
					break;
				case "Other":
					cpType = CustomPropertyDbExAssetType.Other;
					break;
			}

			// return a new object
			return new CustomPropertyDbEx(id, cpType, assetID, 
				r[CustomPropertyTableField.PropertyKey.ToString()].ToString(), 
				r[CustomPropertyTableField.PropertyValue.ToString()].ToString());
		}


		/// <summary>
		/// Convert a dataset to a <see cref="CustomPropertyDbExCollection" />.
		/// </summary>
		/// <param name="ds"><see cref="DataSet" /></param>
		/// <returns><see cref="CustomPropertyDbExCollection" /></returns>
		internal static CustomPropertyDbExCollection ToCustomPropertyDbEx (DataSet ds)
		{
			CustomPropertyDbEx obj;
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			// step through all records in the dataset & add to collection
			foreach (DataRow r in ds.Tables[0].Rows)
			{
				obj = ToCustomPropertyDbEx(r);
				collection.Add(obj);
			}
			
			return collection;
		}
	}
}