#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
/// Microsoft MCMS2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for ChannelBrowser.
	/// </summary>
	public class ChannelBrowser : System.Web.UI.Page
	{
		#region Constants
		#endregion

		#region Objects
		private CmsAssetCollection m_channelCollection;
		#endregion
		
		#region Designer Objects
		protected NestedList ChannelBrowserList;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Populates the CMS channel structure
		/// </summary>
		private void GetChannelStructure()
		{
			Channel channel;
			CmsAsset cmsAsset;

			// create collection object to hold collection
			this.m_channelCollection = new CmsAssetCollection();

			// authenticate into cms
			CmsApplicationContext cmsContext = new CmsApplicationContext();
			WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
			cmsContext.AuthenticateUsingUserHandle(identity.Token, PublishingMode.Published);


			// get the root channel
			channel = (Channel)cmsContext.Searches.GetByPath("/Channels");
			cmsAsset = new CmsAsset();
			cmsAsset.ID = new Guid(channel.Guid.ToString());
			cmsAsset.DisplayName = channel.DisplayName.ToString();
			// if it has subchannels, get them
			cmsAsset.ChildAssets = this.GetChildChannels(channel);


			// add it to the collection
			this.m_channelCollection.Add(cmsAsset);
		}


		/// <summary>
		/// Converts all subchannels in a provided channel to a CmsAssetCollection.
		/// </summary>
		/// <param name="channel"></param>
		/// <returns></returns>
		private CmsAssetCollection GetChildChannels(Channel channel)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();
			
			if (channel.Channels.Count >0)
				foreach (Channel c in channel.Channels){
					asset = new CmsAsset();
					asset.ID = new Guid(c.Guid.ToString());
					asset.DisplayName = c.DisplayName.ToString();
					assetCollection.Add(asset);

					// if there are child channels, add them
					if (c.Channels.Count >0)
						asset.ChildAssets = this.GetChildChannels(c);
				}
			else
				assetCollection = null;

			return assetCollection;
		}



		/// <summary>
		/// Builds a set of tree nodes for the channels
		/// </summary>
		private void BindChannelTree()
		{
			this.ChannelBrowserList.ObjectCollection = this.m_channelCollection;
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			// Put user code to initialize the page here
			if (!this.IsPostBack)
			{
				this.GetChannelStructure();
				this.BindChannelTree();
			}
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.ID = "ChannelTree";
			this.Load += new EventHandler(Page_Load);
		}
		#endregion
	}
}
