#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
/// Microsoft IE web controls
using Microsoft.Web.UI.WebControls;
/// Microsoft MCMS2002 namespaces
using Microsoft.ContentManagement.Publishing;
/// CustomPropertyDbEx namespaces
using ContentManagement.Extensions;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for CmsAssetBrowser.
	/// </summary>
	public class CmsAssetBrowser : System.Web.UI.Page
	{
		#region Constants
		private const string			c_currentPublishingModeQueryStringKey = "pmode";
		private const string			c_currentTreeModeQueryStringKey = "tmode";
		#endregion

		#region Objects
		private PublishingMode			m_currentPublishingMode;
		private string							m_currentTreeMode;
		private CmsAssetCollection	m_collection = null;
		#endregion
		
		#region Designer Objects
		protected Microsoft.Web.UI.WebControls.TreeView HierarchyTreeView;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Get the current publishing mode based off the query string.
		/// </summary>
		private void SetCurrentPublishingMode()
		{
			string key = this.Request.QueryString[c_currentPublishingModeQueryStringKey];
			// if it was specified...
			if (key != null && key.Length >0)
				switch (key.ToString().ToLower())
				{
					case "published":
						this.m_currentPublishingMode = PublishingMode.Published;
						break;
					case "staging":
						this.m_currentPublishingMode = PublishingMode.Staging;
						break;
					case "unpublished":
						this.m_currentPublishingMode = PublishingMode.Unpublished;
						break;
					case "update":
						this.m_currentPublishingMode = PublishingMode.Update;
						break;
				}
			else
				this.m_currentPublishingMode = PublishingMode.Published;
		}


		/// <summary>
		/// Get the current tree mode based off the query string.
		/// </summary>
		private void SetCurrentTreeMode()
		{
			string key = this.Request.QueryString[c_currentTreeModeQueryStringKey];
			// if it was specified...
			if (key != null && key.Length >0)
				this.m_currentTreeMode = key.ToString().ToLower();
			else
				this.m_currentTreeMode = "channel";
		}


		/// <summary>
		/// Logs in current user based on the publishing mode.
		/// </summary>
		/// <returns></returns>
		private CmsApplicationContext GetCurrentCmsContext()
		{
			CmsApplicationContext cmsContext = new CmsApplicationContext();
			WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
			cmsContext.AuthenticateUsingUserHandle(identity.Token, this.m_currentPublishingMode);

			return cmsContext;
		}


		/// <summary>
		/// Init's all settings for hte treeview
		/// </summary>
		private void SetTreeViewProperties()
		{
			/// expand the first 4 levels
			HierarchyTreeView.ExpandLevel = 2;

			/// set different types of treenodes
			///
			string imageUrl = "images/";
			TreeNodeType type;

			// create for channels
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.channel";
			type.ImageUrl = imageUrl +"Channel.gif";
			type.ExpandedImageUrl = imageUrl +"Channel.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);

			// create for postings
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.posting";
			type.ImageUrl = imageUrl +"Posting.gif";
			type.ExpandedImageUrl = imageUrl +"Posting.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);

			// create for Template
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.template";
			type.ImageUrl = imageUrl +"Template.gif";
			type.ExpandedImageUrl = imageUrl +"Template.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);

			// create for templateGallery
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.templategallery";
			type.ImageUrl = imageUrl +"TemplateGallery.gif";
			type.ExpandedImageUrl = imageUrl +"TemplateGallery.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);

			// create for resource
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.resource";
			type.ImageUrl = imageUrl +"ResourceGalleryItem.gif";
			type.ExpandedImageUrl = imageUrl +"Resource.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);

			// create for resourceGallery
			type = new TreeNodeType();
			type.Type = "microsoft.contentmanagement.publishing.resourcegallery";
			type.ImageUrl = imageUrl +"ResourceGallery.gif";
			type.ExpandedImageUrl = imageUrl +"ResourceGallery.gif";
			HierarchyTreeView.TreeNodeTypes.Add(type);
		}


		/// <summary>
		/// Build all tree nodes and binds them to the treeview.
		/// </summary>
		private void BuildTree()
		{
			TreeNode parent;
			CmsAsset topLevel;

			if (this.m_collection != null && this.m_collection.Count >0)
			{
				// buuld the hierarchy
				topLevel = this.m_collection[0];
				parent = this.BuildTreeNode(topLevel);
				if (topLevel.ChildAssets != null && topLevel.ChildAssets.Count>0)
					parent = this.GetNodeTree(parent, topLevel);


				// bind it to the treeview
				HierarchyTreeView.Nodes.Add(parent);
			}
		}


		/// <summary>
		/// Builds a tree node based off specified criteria.
		/// </summary>
		/// <returns>Built tree node.</returns>
		private TreeNode BuildTreeNode(CmsAsset asset)
		{
			TreeNode t = new TreeNode();
			t.Text = asset.DisplayName;	
			t.NavigateUrl = string.Format("javascript:Node_OnClick(this,'{0}',{1});", asset.ID.ToString().ToUpper(), asset.CanHaveProperties.ToString().ToLower());
			t.Type = asset.CmsAssetType.ToString().ToLower();
			return t;
		}


		/// <summary>
		/// Cms object to build a node tree for.
		/// </summary>
		/// <param name="node">Node for the current channel structure</param>
		/// <param name="cmsObject">Cms object (channel/template/etc) to use.</param>
		/// <returns></returns>
		private TreeNode GetNodeTree(TreeNode node, CmsAsset cmsObject)
		{
			TreeNode childNode;
			
			// check if it has children... if so, build the tree
			if (cmsObject.ChildAssets != null && cmsObject.ChildAssets.Count >0)
				foreach (CmsAsset c in cmsObject.ChildAssets)
				{
					// build tree
					childNode = this.BuildTreeNode(c);

					// if this one has children, get them
					if (c.ChildAssets != null && c.ChildAssets.Count >0)
						childNode = this.GetNodeTree(childNode, c);

					// add to child collection
					node.Nodes.Add(childNode);
				}

			return node;
		}
		
		
		/// <summary>
		/// Populates the CMS channel structure.
		/// </summary>
		private void LoadChannelStructure()
		{
			Channel channel;
			CmsAsset cmsAsset;

			// create collection object to hold collection
			this.m_collection = new CmsAssetCollection();


			// authenticate into cms
			CmsApplicationContext cmsContext = this.GetCurrentCmsContext();


			// get the root channel
			channel = (Channel)cmsContext.Searches.GetByPath("/Channels");
			cmsAsset = new CmsAsset();
			cmsAsset.ID = new Guid(channel.Guid.ToString());
			cmsAsset.DisplayName = channel.Name.ToString();
			cmsAsset.CmsAssetType = channel.GetType();
			cmsAsset.CanHaveProperties = true;
			// if it has subchannels, get them
			cmsAsset.ChildAssets.AddRange(this.GetChildChannels(channel));
			if (channel.Postings.Count >0)
				cmsAsset.ChildAssets.AddRange(this.GetChildPostings(channel));

			// add it to the collection
			this.m_collection.Add(cmsAsset);
		}


		/// <summary>
		/// Converts all subchannels in a provided channel to a CmsAssetCollection.
		/// </summary>
		/// <param name="channel">CMS channel to traverse.</param>
		/// <returns>Collection of CMS channels.</returns>
		private CmsAssetCollection GetChildChannels(Channel channel)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();
			
			if (channel.Channels.Count >0)
				foreach (Channel c in channel.Channels)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(c.Guid.ToString());
					asset.DisplayName = c.Name.ToString();
					asset.CmsAssetType = c.GetType();
					asset.CanHaveProperties = true;
					assetCollection.Add(asset);
					if (c.Postings.Count >0)
						asset.ChildAssets.AddRange(this.GetChildPostings(c));

					// if there are child channels, add them
					if (c.Channels.Count >0)
						asset.ChildAssets.AddRange(this.GetChildChannels(c));

					asset.DisplayName = c.Name.ToString();
				}
			else
				assetCollection = null;

			return assetCollection;
		}


		/// <summary>
		/// Converts all postings in a provided channel to a CmsAssetCollection.
		/// </summary>
		/// <param name="channel">CMS channel to traverse.</param>
		/// <returns>Collection of CMS postings.</returns>
		private CmsAssetCollection GetChildPostings(Channel channel)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();

			if (channel.Postings.Count >0)
				foreach (Posting p in channel.Postings)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(p.Guid.ToString());
					asset.DisplayName = p.Name;
					asset.CmsAssetType = p.GetType();
					asset.CanHaveProperties = true;
					assetCollection.Add(asset);
				}
			else
				assetCollection = null;

			return assetCollection;
		}


		/// <summary>
		/// Populates the CMS template structure.
		/// </summary>
		private void LoadTemplateStructure()
		{
			TemplateGallery gallery;
			CmsAsset cmsAsset;

			// create collection object to hold collection
			this.m_collection = new CmsAssetCollection();


			// authenticate into cms
			CmsApplicationContext cmsContext = this.GetCurrentCmsContext();


			// get the root template gallery
			gallery = (TemplateGallery)cmsContext.Searches.GetByPath("/Templates");
			cmsAsset = new CmsAsset();
			cmsAsset.ID = new Guid(gallery.Guid.ToString());
			cmsAsset.DisplayName = gallery.Name.ToString();
			cmsAsset.CmsAssetType = gallery.GetType();
			cmsAsset.CanHaveProperties = false;
			// if it has subchannels, get them
			cmsAsset.ChildAssets.AddRange(this.GetChildTemplateGalleries(gallery));
			if (gallery.Templates.Count >0)
				cmsAsset.ChildAssets.AddRange(this.GetChildTemplates(gallery));

			// add it to the collection
			this.m_collection.Add(cmsAsset);
		}


		/// <summary>
		/// Converts all templategalleries in a provided templategallery to a CmsAssetCollection.
		/// </summary>
		/// <param name="templateGallery">CMS TemplateGallery to traverse.</param>
		/// <returns>Collection of CMS TemplateGallery.</returns>
		private CmsAssetCollection GetChildTemplateGalleries(TemplateGallery templateGallery)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();
			
			if (templateGallery.TemplateGalleries.Count >0)
				foreach (TemplateGallery tg in templateGallery.TemplateGalleries)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(tg.Guid.ToString());
					asset.DisplayName = tg.Name.ToString();
					asset.CmsAssetType = tg.GetType();
					asset.CanHaveProperties = false;
					assetCollection.Add(asset);
					if (tg.Templates.Count >0)
						asset.ChildAssets.AddRange(this.GetChildTemplates(tg));

					// if there are child galleries, add them
					if (tg.TemplateGalleries.Count >0)
						asset.ChildAssets.AddRange(this.GetChildTemplateGalleries(tg));

					asset.DisplayName = tg.Name.ToString();
				}
			else
				assetCollection = null;

			return assetCollection;
		}


		/// <summary>
		/// Converts all templates in a provided gallery to a CmsAssetCollection.
		/// </summary>
		/// <param name="templateGallery">CMS templateGallery to traverse.</param>
		/// <returns>Collection of CMS templates.</returns>
		private CmsAssetCollection GetChildTemplates(TemplateGallery templateGallery)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();

			if (templateGallery.Templates.Count >0)
				foreach (Template t in templateGallery.Templates)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(t.Guid.ToString());
					asset.DisplayName = t.Name;
					asset.CmsAssetType = t.GetType();
					asset.CanHaveProperties = true;
					assetCollection.Add(asset);
				}
			else
				assetCollection = null;

			return assetCollection;
		}

	
		/// <summary>
		/// Populates the CMS resource structure.
		/// </summary>
		private void LoadResourceStructure()
		{
			ResourceGallery gallery;
			CmsAsset cmsAsset;

			// create collection object to hold collection
			this.m_collection = new CmsAssetCollection();


			// authenticate into cms
			CmsApplicationContext cmsContext = this.GetCurrentCmsContext();


			// get the root resource gallery
			gallery = (ResourceGallery)cmsContext.Searches.GetByPath("/Resources");
			cmsAsset = new CmsAsset();
			cmsAsset.ID = new Guid(gallery.Guid.ToString());
			cmsAsset.DisplayName = gallery.Name.ToString();
			cmsAsset.CmsAssetType = gallery.GetType();
			cmsAsset.CanHaveProperties = false;
			// if it has subchannels, get them
			cmsAsset.ChildAssets.AddRange(this.GetChildResourceGalleries(gallery));
			if (gallery.Resources.Count >0)
				cmsAsset.ChildAssets.AddRange(this.GetChildResources(gallery));

			// add it to the collection
			this.m_collection.Add(cmsAsset);
		}


		/// <summary>
		/// Converts all resourceGallery in a provided resourceGallery to a CmsAssetCollection.
		/// </summary>
		/// <param name="resourceGallery">CMS ResourceGallery to traverse.</param>
		/// <returns>Collection of CMS ResourceGallery.</returns>
		private CmsAssetCollection GetChildResourceGalleries(ResourceGallery resourceGallery)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();
			
			if (resourceGallery.ResourceGalleries.Count >0)
				foreach (ResourceGallery rg in resourceGallery.ResourceGalleries)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(rg.Guid.ToString());
					asset.DisplayName = rg.Name.ToString();
					asset.CmsAssetType = rg.GetType();
					asset.CanHaveProperties = false;
					assetCollection.Add(asset);
					if (rg.Resources.Count >0)
						asset.ChildAssets.AddRange(this.GetChildResources(rg));

					// if there are child galleries, add them
					if (rg.ResourceGalleries.Count >0)
						asset.ChildAssets.AddRange(this.GetChildResourceGalleries(rg));

					asset.DisplayName = rg.Name.ToString();
				}
			else
				assetCollection = null;

			return assetCollection;
		}


		/// <summary>
		/// Converts all resources in a provided gallery to a CmsAssetCollection.
		/// </summary>
		/// <param name="resourceGallery">CMS resourceGallery to traverse.</param>
		/// <returns>Collection of CMS resources.</returns>
		private CmsAssetCollection GetChildResources(ResourceGallery resourceGallery)
		{
			CmsAsset asset;
			CmsAssetCollection assetCollection = new CmsAssetCollection();

			if (resourceGallery.Resources.Count >0)
				foreach (Resource r in resourceGallery.Resources)
				{
					asset = new CmsAsset();
					asset.ID = new Guid(r.Guid.ToString());
					asset.DisplayName = r.Name;
					asset.CmsAssetType = r.GetType();
					asset.CanHaveProperties = true;
					assetCollection.Add(asset);
				}
			else
				assetCollection = null;

			return assetCollection;
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			// set the publishing mode
			this.SetCurrentPublishingMode();

			// set the current tree mode
			this.SetCurrentTreeMode();

			// set the tree properties
			this.SetTreeViewProperties();

			switch (this.m_currentTreeMode)
			{
				case "channel":
					this.LoadChannelStructure();
					break;
				case "template":
					this.LoadTemplateStructure();
					break;
				case "resource":
					this.LoadResourceStructure();
					break;
				case "other":
					break;
			}

			this.BuildTree();
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}

		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}