#region Assembly references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.Xml;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for CmsAssetCollection.
	/// </summary>
	[Serializable]
	public class CmsAssetCollection : CollectionBase
	{
		#region Constructor
		/// <summary>
		/// Inits a new instance of the CmsAssetCollection class
		/// </summary>
		public CmsAssetCollection(){}

		
		/// <summary>
		/// Inits a new instance of the CmsAssetCollection class
		/// </summary>
		public CmsAssetCollection(CmsAssetCollection value)
		{
			this.AddRange(value);
		}

		
		/// <summary>
		/// Inits a new instance of the CmsAssetCollection class
		/// </summary>
		public CmsAssetCollection(CmsAsset[] value)
		{
			this.AddRange(value);
		}
		#endregion

		#region Public class methods
		/// <summary>
		/// Adds the CmsAsset object to this collections
		/// </summary>
		/// <param name="value"></param>
		/// <returns></returns>
		public int Add(CmsAsset value)
		{
			return this.List.Add(value);
		}


		/// <summary>
		/// Copies the elements of the specified CmsAsset array to the end of the collection.
		/// </summary>
		/// <param name="value">An array of type CmsAsset containing the Components to add to the collection.</param>
		public void AddRange(CmsAsset[] value) 
		{
			for (int i = 0;	(i < value.Length); i = (i + 1)) 
			{
				this.Add(value[i]);
			}
		}


		/// <summary>
		/// Adds the contents of another CmsAssetCollection to the end of the collection.
		/// </summary>
		/// <param name="value">A CmsAssetCollection containing the Components to add to the collection. </param>
		public void AddRange(CmsAssetCollection value) 
		{
			for (int i = 0;	(i < value.Count); i = (i +	1))	
			{
				this.Add((CmsAsset)value.List[i]);
			}
		}
		

		/// <summary>
		/// Gets a value indicating whether the collection contains the specified CmsAssetCollection.
		/// </summary>
		/// <param name="value">The CmsAssetCollection to search for in the collection.</param>
		/// <returns><b>true</b> if the collection contains the specified object; otherwise, <b>false</b>.</returns>
		public bool Contains(CmsAsset value) 
		{
			return this.List.Contains(value);
		}
		
		
		/// <summary>
		/// Copies the collection Components to a one-dimensional <see cref="T:System.Array">Array</see> instance beginning at the specified index.
		/// </summary>
		/// <param name="array">The one-dimensional <see cref="T:System.Array">Array</see> that is the destination of the values copied from the collection.</param>
		/// <param name="index">The index of the array at which to begin inserting.</param>
		public void CopyTo(CmsAsset[] array, int index) 
		{
			this.List.CopyTo(array, index);
		}
		

		/// <summary>
		/// Gets the index in the collection of the specified CmsAssetCollection, if it exists in the collection.
		/// </summary>
		/// <param name="value">The CmsAssetCollection to locate in the collection.</param>
		/// <returns>The index in the collection of the specified object, if found; otherwise, -1.</returns>
		public int IndexOf(CmsAsset value) 
		{
			return this.List.IndexOf(value);
		}
		

		/// <summary>
		/// Inserts the specified postings into the specified index
		/// </summary>
		/// <param name="index">index where the new element should be added</param>
		/// <param name="value">element to insert</param>
		public void Insert(int index, CmsAsset value)	
		{
			List.Insert(index, value);
		}
	
		
		/// <summary>
		/// Removes the specified element from the collection
		/// </summary>
		/// <param name="value">element to remove from the collection</param>
		public void Remove(CmsAsset value) 
		{
			List.Remove(value);
		}
		

		/// <summary>
		/// Returns an enumerator that can iterate through the CmsAssetCollection instance.
		/// </summary>
		/// <returns>An CmsAssetCollection for the CmsAssetCollection instance.</returns>
		public new CmsAssetCollectionEnumerator GetEnumerator()	
		{
			return new CmsAssetCollectionEnumerator(this);
		}
		
		#endregion

		#region Enumeration construct
		/// <summary>
		/// Supports a simple iteration over a CmsAssetCollection.
		/// </summary>
		public class CmsAssetCollectionEnumerator : IEnumerator	
		{
			private	IEnumerator _enumerator;
			private	IEnumerable _temp;
			
			/// <summary>
			/// Initializes a new instance of the CmsAssetCollectionEnumerator class referencing the specified object.
			/// </summary>
			/// <param name="mappings">The CmsAssetCollection to enumerate.</param>
			public CmsAssetCollectionEnumerator(CmsAssetCollection mappings)
			{
				_temp =	((IEnumerable)(mappings));
				_enumerator = _temp.GetEnumerator();
			}
			
			/// <summary>
			/// Gets the current element in the collection.
			/// </summary>
			public CmsAsset Current
			{
				get {return ((CmsAsset)(_enumerator.Current));}
			}
			
			object IEnumerator.Current
			{
				get {return _enumerator.Current;}
			}
			
			/// <summary>
			/// Advances the enumerator to the next element of the collection.
			/// </summary>
			/// <returns><b>true</b> if the enumerator was successfully advanced to the next element; <b>false</b> if the enumerator has passed the end of the collection.</returns>
			public bool MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			/// <summary>
			/// Sets the enumerator to its initial position, which is before the first element in the collection.
			/// </summary>
			public void Reset()
			{
				_enumerator.Reset();
			}
			
			void IEnumerator.Reset() 
			{
				_enumerator.Reset();
			}
		}
		#endregion

		#region Public class properties
		/// <summary>
		/// Gets the CmsAssetCollection at the specified index in the collection.
		/// <para>
		/// In C#, this property is the indexer for the CmsAssetCollection class.
		/// </para>
		/// </summary>
		public CmsAsset this[int index] 
		{
			get	{return ((CmsAsset)(this.List[index]));}
		}
		#endregion
	}
}