#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.Xml;
/// Microsoft CMS 2002 namespaces
using Microsoft.ContentManagement.Publishing;
#endregion

namespace ContentManagement.Extensions
{
	/// <summary>
	/// Overloaded and extended <see cref="CollectionBase"/> used to handle the collection of <see cref="CustomPropertyDbEx"/>.
	/// </summary>
	[Serializable]
	public class CustomPropertyDbExCollection : CollectionBase
	{
		#region Constructor
		/// <overloads>
		/// Inialiteze a new instance of the <see cref="CustomPropertyDbExCollection"/> class
		/// </overloads>
		/// <summary>
		/// Creates a new, empty <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		public CustomPropertyDbExCollection(){}

		
		/// <summary>
		/// Creates a new <see cref="CustomPropertyDbExCollection"/> filled with the provided collection.
		/// </summary>
		/// <param name="value"><see cref="CustomPropertyDbExCollection"/> to prepopulate in this new collection.</param>
		public CustomPropertyDbExCollection(CustomPropertyDbExCollection value)
		{
			this.AddRange(value);
		}

		
		/// <summary>
		/// Creates a new <see cref="CustomPropertyDbExCollection"/> filled with the provided collection.
		/// </summary>
		/// <param name="value"><see cref="Array"/> of <see cref="CustomPropertyDbEx"/> to prepopulate in this new collection.</param>
		public CustomPropertyDbExCollection(CustomPropertyDbEx[] value)
		{
			this.AddRange(value);
		}
		#endregion

		#region Public class methods
		/// <summary>
		/// Adds the <see cref="CustomPropertyDbEx"/> to the end of the <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IList.Add"/>.
		/// </remarks>
		/// <param name="value"><see cref="CustomPropertyDbEx"/> to add to the end of the <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>The <see cref="CollectionBase"/> index at which the <paramref>value</paramref> was added.</returns>
		public int Add(CustomPropertyDbEx value)
		{
			return this.List.Add(value);
		}


		/// <summary>
		/// Copies the elements of the specified <see cref="CustomPropertyDbEx"/> array to the end of the <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <param name="value">An <see cref="Array"/> of type <see cref="CustomPropertyDbEx"/> containing the objects to add to the <see cref="CustomPropertyDbExCollection"/>.</param>
		public void AddRange(CustomPropertyDbEx[] value) 
		{
			for (int i = 0;	(i < value.Length); i = (i + 1)) 
			{
				this.Add(value[i]);
			}
		}


		/// <summary>
		/// Copies the elements of the specified <see cref="CustomPropertyDbEx"/> array to the end of the <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <param name="value">A <see cref="CustomPropertyDbExCollection"/> containing the objects to add to the <see cref="CustomPropertyDbExCollection"/>.</param>
		public void AddRange(CustomPropertyDbExCollection value) 
		{
			for (int i = 0;	(i < value.Count); i = (i +	1))	
			{
				this.Add((CustomPropertyDbEx)value.List[i]);
			}
		}
		

		/// <summary>
		/// Determines whether the <see cref="CollectionBase"/> contains a specific element.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IList.Contains"/>.
		/// </remarks>
		/// <param name="value">The <see cref="CustomPropertyDbExCollection"/> to locate for in the <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns><b>true</b> if the <see cref="CustomPropertyDbExCollection"/> contains the specified object; otherwise, <b>false</b>.</returns>
		public bool Contains(CustomPropertyDbEx value) 
		{
			return this.List.Contains(value);
		}


		/// <summary>
		/// Searches for the specified <see cref="CustomPropertyDbEx"/> and returns the zero-based index of the first 
		/// occurrence within the entire <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IList.IndexOf"/>.
		/// </remarks>
		/// <param name="value">The <see cref="CustomPropertyDbEx"/> to locate in the <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>The index in the <see cref="CustomPropertyDbExCollection"/> of the specified object, if found; otherwise, -1.</returns>
		public int IndexOf(CustomPropertyDbEx value) 
		{
			return this.List.IndexOf(value);
		}
		

		/// <summary>
		/// Inserts an <see cref="CustomPropertyDbEx"/> into the <see cref="CustomPropertyDbExCollection"/> at the specified index.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IList.Insert"/>.
		/// </remarks>
		/// <param name="index">The zero-based index at which value should be inserted.</param>
		/// <param name="value"><see cref="CustomPropertyDbEx"/> to insert.</param>
		public void Insert(int index, CustomPropertyDbEx value)	
		{
			List.Insert(index, value);
		}
	
		
		/// <summary>
		/// Removes the first occurrence of a specific <see cref="CustomPropertyDbEx"/> from the <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IList.Remove"/>.
		/// </remarks>
		/// <param name="value"><see cref="CustomPropertyDbEx"/> to remove from the <see cref="CustomPropertyDbExCollection"/>.</param>
		public void Remove(CustomPropertyDbEx value) 
		{
			List.Remove(value);
		}
		

		/// <summary>
		/// Returns an enumerator that can iterate through the <see cref="CustomPropertyDbExCollection"/> instance.
		/// </summary>
		/// <remarks>
		/// 	<para>
		/// 	Enumerators only allow reading the data in the <see cref="CustomPropertyDbExCollection"/>. Enumerators cannot be used to 
		/// 	modify the underlying <see cref="CustomPropertyDbExCollection"/>.
		/// 	</para>
		/// 	<para>
		/// 	Initially, the enumerator is positioned before the first element in the collection. Reset also brings the enumerator back 
		/// 	to this position. At this position, calling <b>Current</b> throws an exception. Therefore, you must call <b>MoveNext</b> to 
		/// 	advance the enumerator to the first element of the <see cref="CustomPropertyDbExCollection"/> before reading the value of 
		/// 	<b>Current</b>.
		/// 	</para>
		/// 	<para>
		/// 	Current returns the same object until either <b>MoveNext</b> or <b>Reset</b> is called. <b>MoveNext</b> sets <b>Current</b> to the 
		/// 	next element.
		/// 	</para>
		/// 	<para>
		/// 	After the end of the <see cref="CustomPropertyDbExCollection"/> is passed, the enumerator is positioned after the last element in 
		/// 	the <see cref="CustomPropertyDbExCollection"/>, and calling <b>MoveNext</b> returns <b>false</b>. If the last call to <b>MoveNext</b> 
		/// 	returned <b>false</b>, calling <b>Current</b> throws an exception. To set <b>Current</b> to the first element of the 
		/// 	<see cref="CustomPropertyDbExCollection"/> again, you can call <b>Reset</b> followed by <b>MoveNext</b>.
		/// 	</para>
		/// 	<para>
		/// 	An enumerator remains valid as long as the <see cref="CustomPropertyDbExCollection"/> remains unchanged. If changes are made to 
		/// 	the <see cref="CustomPropertyDbExCollection"/>, such as adding, modifying or deleting elements, the enumerator is irrecoverably 
		/// 	invalidated and the next call to <b>MoveNext</b> or <b>Reset</b> throws an <see cref="InvalidOperationException"/>. If the 
		/// 	<see cref="CustomPropertyDbExCollection"/> is modified between <b>MoveNext</b> and <b>Current</b>, <b>Current</b> will return the 
		/// 	element that it is set to, even if the enumerator is already invalidated.
		/// 	</para>
		/// 	<para>
		/// 	The enumerator does not have exclusive access to the <see cref="CustomPropertyDbExCollection"/>; therefore, enumerating through 
		/// 	a <see cref="CustomPropertyDbExCollection"/> is intrinsically not a thread-safe procedure. Even when a 
		/// 	<see cref="CustomPropertyDbExCollection"/> is synchronized, other threads could still modify the 
		/// 	<see cref="CustomPropertyDbExCollection"/>, which causes the enumerator to throw an exception. To guarantee thread safety during 
		/// 	enumeration, you can either lock the <see cref="CustomPropertyDbExCollection"/> during the entire enumeration or catch the exceptions 
		/// 	resulting from changes made by other threads.
		/// 	</para>		
		/// </remarks>
		/// <returns>An <see cref="IEnumerator"/> for the <see cref="CustomPropertyDbExCollection"/> instance.</returns>
		public new CustomPropertyDbExCollectionEnumerator GetEnumerator()	
		{
			return new CustomPropertyDbExCollectionEnumerator(this);
		}
		
		
		/// <summary>
		/// Calls <see cref="CustomPropertyDbEx.Commit()"/> on all items in the <see cref="CustomPropertyDbExCollection"/> 
		/// to the database.
		/// </summary>
		public void Commit()
		{
			// call Commit on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Commit();
		}


		/// <summary>
		/// Calls <see cref="CustomPropertyDbEx.Sync()"/> on all items in the <see cref="CustomPropertyDbExCollection"/> 
		/// with the database (so they have the same contents as what's in the database.
		/// </summary>
		public void Sync()
		{
			// call Sync on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Sync();
		}


		/// <summary>
		/// Calls <see cref="CustomPropertyDbEx.Delete()"/> on all items in the <see cref="CustomPropertyDbExCollection"/> 
		/// to the database.
		/// </summary>
		/// <remarks>
		/// Must call <see cref="CustomPropertyDbEx.Commit()"/> to take effect in the database.
		/// </remarks>
		public void Delete()
		{
			// call Delete on each object
			foreach(object o in this.List)
				((CustomPropertyDbEx)o).Delete();
		}


		/// <summary>
		/// Creates a copy of the current <see cref="CustomPropertyDbExCollection"/> and specifies the 
		/// <paramref>newAssetType</paramref> & <paramref>newAssetGuid</paramref> for the items in the new collection 
		/// with the provided items.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include <see cref="CustomPropertyDbEx.Clone()"/>.</b>  You must call <see cref="CustomPropertyDbExCollection.CommitAll()"/> 
		/// or <see cref="CustomPropertyDbExCollection.Commit()"/> on the collection or on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="newAssetType">Type of asset for new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <param name="newAssetGuid">Unique ID of asset for new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>Copy of the current <see cref="CustomPropertyDbExCollection"/>, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(CustomPropertyDbExAssetType newAssetType, Guid newAssetGuid)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(newAssetType, newAssetGuid));

			return collection;
		}


		/// <summary>
		/// Creates a copy of the current <see cref="CustomPropertyDbExCollection"/> and specifies the <paramref>channel</paramref>
		/// for the items in the new <see cref="CustomPropertyDbExCollection"/> with the provided items.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include <see cref="CustomPropertyDbEx.Clone()"/>.</b>  You must call <see cref="CustomPropertyDbExCollection.CommitAll()"/> 
		/// or <see cref="CustomPropertyDbExCollection.Commit()"/> on the collection or on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="channel">CMS channel for the new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>Copy of the current <see cref="CustomPropertyDbExCollection"/>, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(Channel channel)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(channel));

			return collection;
		}


		/// <summary>
		/// Creates a copy of the current <see cref="CustomPropertyDbExCollection"/> and specifies the <paramref>template</paramref>
		/// for the items in the new <see cref="CustomPropertyDbExCollection"/> with the provided items.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include <see cref="CustomPropertyDbEx.Clone()"/>.</b>  You must call <see cref="CustomPropertyDbExCollection.CommitAll()"/> 
		/// or <see cref="CustomPropertyDbExCollection.Commit()"/> on the collection or on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="template">CMS template for the new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>Copy of the current <see cref="CustomPropertyDbExCollection"/>, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(Template template)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(template));

			return collection;
		}


		/// <summary>
		/// Creates a copy of the current <see cref="CustomPropertyDbExCollection"/> and specifies the <paramref>resource</paramref>
		/// for the items in the new <see cref="CustomPropertyDbExCollection"/> with the provided items.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include <see cref="CustomPropertyDbEx.Clone()"/>.</b>  You must call <see cref="CustomPropertyDbExCollection.CommitAll()"/> 
		/// or <see cref="CustomPropertyDbExCollection.Commit()"/> on the collection or on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="resource">CMS resource for the new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>Copy of the current <see cref="CustomPropertyDbExCollection"/>, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(Resource resource)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(resource));

			return collection;
		}


		/// <summary>
		/// Creates a copy of the current <see cref="CustomPropertyDbExCollection"/> and specifies the <paramref>posting</paramref>
		/// for the items in the new <see cref="CustomPropertyDbExCollection"/> with the provided items.
		/// </summary>
		/// <remarks>
		/// This method is commonly used for copying custom properties from one CMS asset to another.<br>
		/// <b>Does not include <see cref="CustomPropertyDbEx.Clone()"/>.</b>  You must call <see cref="CustomPropertyDbExCollection.CommitAll()"/> 
		/// or <see cref="CustomPropertyDbExCollection.Commit()"/> on the collection or on each object in the collection to save changes to the database.
		/// </remarks>
		/// <param name="posting">CMS posting for the new <see cref="CustomPropertyDbExCollection"/>.</param>
		/// <returns>Copy of the current <see cref="CustomPropertyDbExCollection"/>, but with a specific asset type and asset id changed for all properties.</returns>
		public CustomPropertyDbExCollection Clone(Posting posting)
		{
			CustomPropertyDbExCollection collection = new CustomPropertyDbExCollection();

			foreach(CustomPropertyDbEx cp in this.List)
				collection.Add(cp.Clone(posting));

			return collection;
		}


		/// <summary>
		/// Sorts the <see cref="CustomPropertyDbExCollection"/> by the <see cref="CustomPropertyDbEx.AssetType"/>, 
		/// <see cref="CustomPropertyDbEx.AssetGuid"/>, then <see cref="CustomPropertyDbEx.Key"/>, then <see cref="CustomPropertyDbEx.Value"/>.
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void Sort(SortDirection direction)
		{
			//TODO: implement this
		}


		/// <summary>
		/// Sorts the <see cref="CustomPropertyDbExCollection"/> by the <see cref="CustomPropertyDbEx.Key"/>.
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void SortByKey(SortDirection direction)
		{
			//TODO: implement this
		}


		/// <summary>
		/// Sorts the <see cref="CustomPropertyDbExCollection"/> by the <see cref="CustomPropertyDbEx.Value"/>.
		/// </summary>
		/// <param name="direction">Direction to sort the collection</param>
		public void SortByValue(SortDirection direction)
		{
			//TODO: implement this
		}
		#endregion

		#region Enumeration construct
		/// <summary>
		/// Supports a simple iteration over a <see cref="CustomPropertyDbExCollection"/>.
		/// </summary>
		/// <remarks>
		/// Implements <see cref="IEnumerator"/>.
		/// </remarks>
		public class CustomPropertyDbExCollectionEnumerator : IEnumerator	
		{
			private	IEnumerator _enumerator;
			private	IEnumerable _temp;
			
			/// <summary>
			/// Initializes a new instance of the CustomPropertyDbExCollectionEnumerator class referencing the specified object.
			/// </summary>
			/// <param name="mappings">The CustomPropertyDbExCollection to enumerate.</param>
			public CustomPropertyDbExCollectionEnumerator(CustomPropertyDbExCollection mappings)
			{
				_temp =	((IEnumerable)(mappings));
				_enumerator = _temp.GetEnumerator();
			}
			
			/// <summary>
			/// Gets the current element in the <see cref="CustomPropertyDbExCollection"/>.
			/// </summary>
			public CustomPropertyDbEx Current
			{
				get {return ((CustomPropertyDbEx)(_enumerator.Current));}
			}
			
			object IEnumerator.Current
			{
				get {return _enumerator.Current;}
			}
			
			/// <summary>
			/// Advances the enumerator to the next element of the <see cref="CustomPropertyDbExCollection"/>.
			/// </summary>
			/// <returns><b>True</b> if the enumerator was successfully advanced to the next element; <b>false</b> if the enumerator has passed the end of the collection.</returns>
			public bool MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return _enumerator.MoveNext();
			}
			
			/// <summary>
			/// Sets the enumerator to its initial position, which is before the first element in the <see cref="CustomPropertyDbExCollection"/>.
			/// </summary>
			public void Reset()
			{
				_enumerator.Reset();
			}
			
			void IEnumerator.Reset() 
			{
				_enumerator.Reset();
			}
		}
		#endregion

		#region Public class properties
		/// <summary>
		/// Gets the <see cref="CustomPropertyDbExCollection"/> at the specified index in the <see cref="CustomPropertyDbExCollection"/>.
		/// <para>
		/// In C#, this property is the indexer for the <see cref="CustomPropertyDbExCollection"/> class.
		/// </para>
		/// </summary>
		/// <value>
		/// Returns the <see cref="CustomPropertyDbEx"/> at the specified index.
		/// </value>
		public CustomPropertyDbEx this[int index] 
		{
			get	{return ((CustomPropertyDbEx)(this.List[index]));}
		}
		#endregion
	}
}