#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.IO;
using System.Data;
using System.Web;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for CustomPropertyTemplate.
	/// </summary>
	internal class CustomPropertyTemplate
	{
		#region Constants
		private const string			c_templateFileName = "CpDbExTemplate.xml";
		#endregion

		#region Objects
		private CpDbExTemplate		m_cpDataSet;
		#endregion

		#region Constructor & Descructor
		/// <summary>
		/// Initializes the object by loading an existing template, or creating a new one.
		/// </summary>
		public CustomPropertyTemplate()
		{
			this.m_cpDataSet = this.LoadTemplate(HttpContext.Current.Request.MapPath(c_templateFileName));

			if (this.m_cpDataSet == null)
				this.m_cpDataSet = new CpDbExTemplate();
		}
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Attempts to load a template file from the provided path.
		/// </summary>
		/// <param name="path">Full path to the file to load.</param>
		/// <returns>Loaded template object or null if the template file couldn't be loaded.</returns>
		private CpDbExTemplate LoadTemplate(string path)
		{
			if (!File.Exists(path))
				return null;

			try
			{
				CpDbExTemplate t = new CpDbExTemplate();
				t.ReadXml(path);
				t.AcceptChanges();
				return t;
			}
			catch
			{return null;}
		}
		#endregion

		#region Public methods
		/// <summary>
		/// Saves all changes to the server.
		/// </summary>
		public void SaveChanges()
		{
			if (this.m_cpDataSet != null)
				this.m_cpDataSet.WriteXml(HttpContext.Current.Request.PhysicalApplicationPath +c_templateFileName);
		}
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		#endregion
	}
}