#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
/// Microsoft MCMS2002 namespaces
using Microsoft.ContentManagement.Publishing;
/// CustomPropertyDbEx namespaces
using ContentManagement.Extensions;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for Footer.
	/// </summary>
	public class Footer : System.Web.UI.Page
	{
		#region Constants
		#endregion

		#region Objects
		#endregion
		
		#region Designer Objects
		protected System.Web.UI.WebControls.Literal CmsObjectName;
		protected System.Web.UI.WebControls.Literal CmsObjectDisplayName;
		protected System.Web.UI.WebControls.Literal CmsObjectGuid;
		protected System.Web.UI.WebControls.Literal CmsObjectPath;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Get a reference to the specified Cms object.
		/// </summary>
		/// <param name="guid">Unique ID of CMS Hierarchy Item.</param>
		/// <returns>Found hierarchy item.</returns>
		private HierarchyItem GetCmsObject(string guid)
		{
			// authenticate into cms
			CmsApplicationContext cmsContext = new CmsApplicationContext();
			WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
			cmsContext.AuthenticateUsingUserHandle(identity.Token, PublishingMode.Published);

			// get object
			HierarchyItem h = cmsContext.Searches.GetByGuid("{" +guid +"}");

			if (h == null)
				throw new Exception("No CMS object found for specified GUID");

			// make sure it's only a channel/posting/template/resource
			if (h is Channel ||
				h is Posting ||
				h is Template ||
				h is Resource)
				return h;
			else
				return null;
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			string guid = this.Request.QueryString["id"];
			if (guid != null && guid.Length > 0)
			{
				HierarchyItem h = this.GetCmsObject(guid);

				// if valid cms object found
				if (h != null)
				{
					this.CmsObjectName.Text = h.Name.ToString();
					this.CmsObjectGuid.Text = h.Guid.ToString().ToUpper();
					this.CmsObjectPath.Text = h.Path.ToString();
					
					if (h is Channel)
						this.CmsObjectDisplayName.Text = ((Channel)h).DisplayName.ToString();
					else if (h is Posting)
						this.CmsObjectDisplayName.Text = ((Posting)h).DisplayName.ToString();
					else if (h is Template)
						this.CmsObjectDisplayName.Text = ((Template)h).Name.ToString();
					else if (h is Resource)
						this.CmsObjectDisplayName.Text = ((Resource)h).DisplayName.ToString();
				}
				else
					this.CmsObjectDisplayName.Text = "<em>unknown</em>";
			}
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
