#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	///		Summary description for HierarchyList.
	/// </summary>
	public class HierarchyTree : System.Web.UI.UserControl
	{
		#region Objects
		private CmsAssetCollection	m_collection = null;
		#endregion
		
		#region Designer Objects
		protected System.Web.UI.WebControls.Repeater HierarchyTreeRepeater;
		#endregion

		#region Public properties
		/// <summary>
		/// Collection of objects to display.
		/// </summary>
		public CmsAssetCollection ObjectCollection
		{
			get{return this.m_collection;}
			set{this.m_collection = value;}
		}
		#endregion

		#region Event Handlers
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender (e);

			// make sure we at least have address of the server:
			if (this.m_collection == null)
				this.HierarchyTreeRepeater.Visible = false;
			else
			{
				this.HierarchyTreeRepeater.Visible = true;
				this.HierarchyTreeRepeater.DataSource = this.m_collection;
				this.HierarchyTreeRepeater.DataBind();
			}
		}


		/// <summary>
		/// Fired when an item is bound to the HierarchyTreeRepeater Repeater.
		/// </summary>
		/// <param name="sender">Object firing the event.</param>
		/// <param name="e"></param>
		protected void HierarchyTreeRepeater_DoItemDataBound(object sender, RepeaterItemEventArgs e)
		{
			// data item
			CmsAsset node;

			// controls in repeater
			HyperLink hLink;
			PlaceHolder childNodes;
	
			// if the item firing event is part of the data portion of the list...
			if ( (e.Item.ItemType == ListItemType.Item) ||
				(e.Item.ItemType == ListItemType.AlternatingItem) )
			{
				// get data item
				node = (CmsAsset)e.Item.DataItem;

				// get conrols in the repeater
				hLink = (HyperLink)e.Item.FindControl("Node");
				hLink.Text = node.DisplayName;
				string js = string.Format("javascript:LoadProperties('{0}',{1});", node.ID.ToString().ToUpper(), node.CanHaveProperties.ToString().ToLower());
				hLink.NavigateUrl = js;

				if (node.ChildAssets != null && node.ChildAssets.Count > 0)
				{
					HierarchyTree c = (HierarchyTree)this.LoadControl("~/HierarchyTree.ascx");
					c.ObjectCollection = node.ChildAssets;

					childNodes = (PlaceHolder)e.Item.FindControl("ChildNodes");
					childNodes.Controls.Add(c);
				}
			}
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}
		

		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{}
		#endregion
	}
}
