#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
/// Microsoft MCMS2002 namespaces
using Microsoft.ContentManagement.Publishing;
/// CustomPropertyDbEx namespaces
using ContentManagement.Extensions;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for PropertyActionPanel.
	/// </summary>
	public class PropertyActionPanel : System.Web.UI.Page
	{
		#region Constants
		private const string c_querystringValue = "cid";
		#endregion

		#region Objects
		private string												m_guid = string.Empty;
		private CustomPropertyDbExCollection	m_cpCollection;
		private HierarchyItem									m_cmsObject;
		#endregion
		
		#region Designer Objects
		protected System.Web.UI.WebControls.CheckBox TemplateWithData;
		protected System.Web.UI.WebControls.DropDownList TemplateListDropDown;
		protected System.Web.UI.WebControls.LinkButton SaveTemplateLinkButton;
		protected System.Web.UI.WebControls.TextBox TemplateNameTextBox;
		protected System.Web.UI.WebControls.LinkButton ClearValuesLinkButton;
		protected System.Web.UI.WebControls.LinkButton RemoveAllLinkButton;
		protected System.Web.UI.WebControls.LinkButton AddFromTemplateLinkButton;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Loads any templates into the drop down that had been previously saved.
		/// </summary>
		private void InitTemplates()
		{
			CustomPropertyTemplate t = new CustomPropertyTemplate();
			if (t.Templates != null)
			{
				DataRow[] dr = t.Templates.Tables[0].Select(string.Empty, "Name ASC");
				this.TemplateListDropDown.DataSource = dr;
				this.TemplateListDropDown.DataValueField = "Guid";
				this.TemplateListDropDown.DataTextField = "Name";
				this.TemplateListDropDown.DataBind();
			}
		}


		/// <summary>
		/// Save the current properties as a template.
		/// </summary>
		private void SaveTemplate()
		{
			CustomPropertyTemplate templateCollection = new CustomPropertyTemplate();
			CpDbExTemplate.PropertyRow property;
			

			// create a new template and add it to the 
			CpDbExTemplate.TemplateRow template = templateCollection.Templates.Template.NewTemplateRow();
			template.Guid = Guid.NewGuid().ToString();
			template.Name = this.TemplateNameTextBox.Text;
			templateCollection.Templates.Template.AddTemplateRow(template);
			

			// step through property collection and add to the template
			if (this.GetPropertyCollection() != null)
				foreach (CustomPropertyDbEx c in this.GetPropertyCollection())
				{
					property = templateCollection.Templates.Property.NewPropertyRow();
					property.Guid = Guid.NewGuid().ToString();
					property.Key = c.Key;
					if (this.TemplateWithData.Checked)
						property.Value = c.Value;
					else
						property.Value = string.Empty;
					property.TemplateRow = template;

					templateCollection.Templates.Property.AddPropertyRow(property);
				}


			// save changes
			templateCollection.Templates.AcceptChanges();
			templateCollection.SaveChanges();

			// rebind the list of templates & display message when page reloads
			if (!this.IsClientScriptBlockRegistered("SaveOkMessage"))
				this.Page.RegisterClientScriptBlock("SaveOkMessage","<script language=JavaScript>alert('Property template saved.');</script>");
			this.InitTemplates();
		}


		/// <summary>
		/// 
		/// </summary>
		private void AddProperties()
		{
			if (this.TemplateListDropDown.SelectedIndex <0)
			{
				if (!this.IsClientScriptBlockRegistered("Error"))
					this.RegisterClientScriptBlock("Error","<script language=JavaScript>alert('You must select a template.');</script>");
				return;
			}

			// get the current object
			CustomPropertyDbExAssetType assetType = CustomPropertyDbExAssetType.Other;
			HierarchyItem hierarchyItem = this.GetCmsObject();
			CustomPropertyDbExCollection cpCollection = new CustomPropertyDbExCollection();
			if (hierarchyItem is Channel)
				assetType = CustomPropertyDbExAssetType.Channel;
			else if (hierarchyItem is Posting)
				assetType = CustomPropertyDbExAssetType.Posting;
			else if (hierarchyItem is Template)
				assetType = CustomPropertyDbExAssetType.Template;
			else if (hierarchyItem is Resource)
				assetType = CustomPropertyDbExAssetType.Resource;


			// get the template properties
			CustomPropertyTemplate templateCollection = new CustomPropertyTemplate();
			CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])templateCollection.Templates.Template.Select("Guid = '" +this.TemplateListDropDown.SelectedValue.ToString() +"'");


			// loop through the properties and add them to the current object
			foreach(CpDbExTemplate.PropertyRow property in template[0].GetPropertyRows())
				cpCollection.Add(new CustomPropertyDbEx(assetType, new Guid(hierarchyItem.Guid), property.Key, property.Value));
			cpCollection.Commit();


			// message that it is done
			if (!this.IsClientScriptBlockRegistered("AddOkMessage"))
				this.Page.RegisterClientScriptBlock("AddOkMessage","<script language=JavaScript>alert('Properties added.'); parent.document.all.CmsObjectPropertyPanel.src=\"PropertyDetail.aspx?cid=" +this.GetGuid() +"\";</script>");
		}
	

		/// <summary>
		/// Loads the selected HierarchyItem if it isn't already loaded.
		/// </summary>
		/// <returns>Loaded Cms HierarchyItem.</returns>
		private HierarchyItem GetCmsObject()
		{
			if (this.m_cmsObject == null)
			{
				// authenticate into cms
				CmsApplicationContext cmsContext = new CmsApplicationContext();
				WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
				cmsContext.AuthenticateUsingUserHandle(identity.Token, PublishingMode.Published);

				// get object
				this.m_cmsObject = cmsContext.Searches.GetByGuid("{" +this.GetGuid() +"}");

				if (this.m_cmsObject == null)
					throw new Exception("No CMS object found for specified GUID");
			}

			return this.m_cmsObject;
		}


		/// <summary>
		/// Gets all <see cref="CustomPropertyDbEx"/>s associated with the specified CMS HierarchyItem.
		/// </summary>
		/// <param name="hierarchyItem">CMS hierarchyItem to get properties for.</param>
		/// <returns><see cref="CustomPropertyDbExCollection"/> of the matching properties.  Null value returned if no properties found.</returns>
		private CustomPropertyDbExCollection GetProperties(HierarchyItem hierarchyItem)
		{
			if (hierarchyItem is Channel)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Channel)hierarchyItem);
			else if (hierarchyItem is Posting)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Posting)hierarchyItem);
			else if (hierarchyItem is Template)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Template)hierarchyItem);
			else if (hierarchyItem is Resource)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Resource)hierarchyItem);
			else
				return null;
		}
		

		/// <summary>
		/// Returns the loaded collection, or loads it if it hasn't been loaded already.
		/// </summary>
		/// <returns>Loaded collection.</returns>
		private CustomPropertyDbExCollection GetPropertyCollection()
		{
			if (this.m_cpCollection == null)
				// get the collection of properties
				this.m_cpCollection = this.GetProperties(this.GetCmsObject());

			return this.m_cpCollection;
		}


		/// <summary>
		/// Pulls the GUID requested off the querystring if it hasn't already been saved locally.
		/// </summary>
		/// <returns>Guid of the channel requested.</returns>
		private string GetGuid()
		{
			if (this.m_guid == null || this.m_guid == string.Empty)
				this.m_guid = this.Request.QueryString[c_querystringValue].ToString();

			if (this.m_guid == null || this.m_guid == string.Empty)
				throw new Exception("No GUID specified when expected.");

			return this.m_guid;
		}


		/// <summary>
		/// Removes all values from all properties for the current object.
		/// </summary>
		private void ClearAllValues()
		{
			// get current object
			HierarchyItem hierarchyItem = this.GetCmsObject();
			CustomPropertyDbExCollection cpCollection = this.GetProperties(hierarchyItem);
			// remove all vales
			foreach (CustomPropertyDbEx c in cpCollection)
				c.Value = string.Empty;
			// commit all changes
			cpCollection.Commit();

			// build script to reload property viewer page & display message
			StringBuilder sb = new StringBuilder();
			sb.Append("<script language=javascript>");
			sb.AppendFormat("parent.document.all.CmsObjectPropertyPanel.src = \"PropertyDetail.aspx?cid={0}\";", this.GetGuid());
			sb.Append("alert('Property values cleared.');");
			sb.Append("</script>");

			this.Page.RegisterClientScriptBlock("ClearAllValues", sb.ToString());
		}


		/// <summary>
		/// Removes all properties from a object.
		/// </summary>
		private void RemoveAllProperties()
		{
			// get current object
			HierarchyItem hierarchyItem = this.GetCmsObject();
			CustomPropertyDbExCollection cpCollection = this.GetProperties(hierarchyItem);
			// get current object
			cpCollection.Delete();
			// commit all changes
			cpCollection.Commit();

			// build script to reload property viewer page & display message
			StringBuilder sb = new StringBuilder();
			sb.Append("<script language=javascript>");
			sb.AppendFormat("parent.document.all.CmsObjectPropertyPanel.src = \"PropertyDetail.aspx?cid={0}\";", this.GetGuid());
			sb.Append("alert('All properties deleted.');");
			sb.Append("</script>");

			this.Page.RegisterClientScriptBlock("RemoveAllProperties", sb.ToString());
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			string guid = this.Request.QueryString[c_querystringValue];
			if (!IsPostBack && guid != null && guid.ToString().Length >0)
			{
				// load the templates
				this.InitTemplates();
	
				// attach event to the save template link
				this.SaveTemplateLinkButton.Attributes.Add("onclick","return GetTemplateName();");
				this.RemoveAllLinkButton.Attributes.Add("onclick", "return confirm('Are you sure you want to remove all properties?  This is not reverseable!');");
				this.ClearValuesLinkButton.Attributes.Add("onclick", "return confirm('Are you sure you want to clear all property values?  This is not reverseable!');");

				// set the default value of the name template if saved
				this.TemplateNameTextBox.Text = this.GetCmsObject().Name.ToString() +" " +"properties";
			}
		}
	

		/// <summary>
		/// Handles the click event when "save template" is clicked
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SaveTemplateLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.SaveTemplate();
		}


		/// <summary>
		/// Handles the click event when "add template properties" is clicked
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void AddFromTemplateLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.AddProperties();
		}


		/// <summary>
		/// Handles the click event when "remove all properties" is clicked
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RemoveAllLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.RemoveAllProperties();
		}


		/// <summary>
		/// Handles the click event when "clear values" is clicked
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ClearValuesLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.ClearAllValues();
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.SaveTemplateLinkButton.Click += new System.EventHandler(this.SaveTemplateLinkButton_DoClick);
			this.AddFromTemplateLinkButton.Click += new System.EventHandler(this.AddFromTemplateLinkButton_DoClick);
			this.ClearValuesLinkButton.Click += new System.EventHandler(this.ClearValuesLinkButton_DoClick);
			this.RemoveAllLinkButton.Click += new System.EventHandler(this.RemoveAllLinkButton_DoClick);
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}