#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
/// Microsoft MCMS2002 namespaces
using Microsoft.ContentManagement.Publishing;
/// CustomPropertyDbEx namespaces
using ContentManagement.Extensions;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for PropertyDetail.
	/// </summary>
	public class PropertyDetail : System.Web.UI.Page
	{
		#region Constants
		private const string c_querystringValue = "cid";
		#endregion

		#region Objects
		private string												m_guid = string.Empty;
		private CustomPropertyDbExCollection	m_cpCollection;
		private HierarchyItem									m_cmsObject;
		#endregion
		
		#region Designer Objects
		protected System.Web.UI.WebControls.DataGrid CustomPropertyDataGrid;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Loads the selected HierarchyItem if it isn't already loaded.
		/// </summary>
		/// <returns>Loaded Cms HierarchyItem.</returns>
		private HierarchyItem GetCmsObject()
		{
			if (this.m_cmsObject == null)
			{
				// authenticate into cms
				CmsApplicationContext cmsContext = new CmsApplicationContext();
				WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
				cmsContext.AuthenticateUsingUserHandle(identity.Token, PublishingMode.Published);

				// get object
				this.m_cmsObject = cmsContext.Searches.GetByGuid("{" +this.GetGuid() +"}");

				if (this.m_cmsObject == null)
					throw new Exception("No CMS object found for specified GUID");
			}

			return this.m_cmsObject;
		}


		/// <summary>
		/// Gets all <see cref="CustomPropertyDbEx"/>s associated with the specified CMS HierarchyItem.
		/// </summary>
		/// <param name="hierarchyItem">CMS hierarchyItem to get properties for.</param>
		/// <returns><see cref="CustomPropertyDbExCollection"/> of the matching properties.  Null value returned if no properties found.</returns>
		private CustomPropertyDbExCollection GetProperties(HierarchyItem hierarchyItem)
		{
			if (hierarchyItem is Channel)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Channel)hierarchyItem);
			else if (hierarchyItem is Posting)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Posting)hierarchyItem);
			else if (hierarchyItem is Template)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Template)hierarchyItem);
			else if (hierarchyItem is Resource)
				return CustomPropertyDbExGlobal.Searches.GetProperties((Resource)hierarchyItem);
			else
				return null;
		}
		

		/// <summary>
		/// Returns the loaded collection, or loads it if it hasn't been loaded already.
		/// </summary>
		/// <returns>Loaded collection.</returns>
		private CustomPropertyDbExCollection GetPropertyCollection()
		{
			if (this.m_cpCollection == null)
				// get the collection of properties
				this.m_cpCollection = this.GetProperties(this.GetCmsObject());

			return this.m_cpCollection;
		}


		/// <summary>
		/// Pulls the GUID requested off the querystring if it hasn't already been saved locally.
		/// </summary>
		/// <returns>Guid of the channel requested.</returns>
		private string GetGuid()
		{
			if (this.m_guid == null || this.m_guid == string.Empty)
				this.m_guid = this.Request.QueryString[c_querystringValue].ToString();

			if (this.m_guid == null || this.m_guid == string.Empty)
				throw new Exception("No GUID specified when expected.");

			return this.m_guid;
		}
	

		/// <summary>
		/// Binds a <see cref="CustomPropertyDbExCollection"/> to the ASP.NET repeater.
		/// </summary>
		private void BindProperties()
		{
			this.CustomPropertyDataGrid.DataSource = this.GetPropertyCollection();
			this.CustomPropertyDataGrid.DataBind();
		}


		/// <summary>
		/// Creates a new <see cref="CustomPropertyDbEx"/> in the datagrid.
		/// </summary>
		/// <param name="objectGuid">Unique ID of the CMS object.</param>
		/// <param name="propertyKey">Property key to create.</param>
		/// <param name="propertyValue">Property value to create.</param>
		private void CreateProperty(string objectGuid, string propertyKey, string propertyValue)
		{
			CustomPropertyDbExAssetType assetType;

			if (this.GetCmsObject() is Channel)
				assetType = CustomPropertyDbExAssetType.Channel;
			else if (this.GetCmsObject() is Posting)
				assetType = CustomPropertyDbExAssetType.Posting;
			else if (this.GetCmsObject() is Template)
				assetType = CustomPropertyDbExAssetType.Template;
			else if (this.GetCmsObject() is Resource)
				assetType = CustomPropertyDbExAssetType.Resource;
			else
				throw new Exception("Invalid object type.");

			CustomPropertyDbEx c = new CustomPropertyDbEx(assetType, new Guid(objectGuid), propertyKey, propertyValue);
			c.Commit();
		}
		

		/// <summary>
		/// Updates the <see cref="CustomPropertyDbEx"/> modified in the datagrid.
		/// </summary>
		/// <param name="propertyGuid">Unique ID of the <see cref="CustomPropertyDbEx"/>.</param>
		/// <param name="propertyKey">Property key to update.</param>
		/// <param name="propertyValue">Property value to update.</param>
		private void UpdateProperty(string propertyGuid, string propertyKey, string propertyValue)
		{
			CustomPropertyDbEx c = CustomPropertyDbExGlobal.Searches.GetByGuid(new Guid(propertyGuid));
			c.Key = propertyKey;
			c.Value = propertyValue;
			c.Commit();
		}
		

		/// <summary>
		/// Deletes the specified <see cref="CustomPropertyDbEx"/>.
		/// </summary>
		/// <param name="propertyGuid">Unique ID of the <see cref="CustomPropertyDbEx"/>.</param>
		private void DeleteProperty(string propertyGuid)
		{
			CustomPropertyDbEx c = CustomPropertyDbExGlobal.Searches.GetByGuid(new Guid(propertyGuid));
			c.Delete();
			c.Commit();
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			string guid = this.Request.QueryString[c_querystringValue];
			if (!IsPostBack && guid != null && guid.ToString().Length >0)
			{
				// bind the collection
				if (this.GetPropertyCollection() != null)
					this.BindProperties();

				// set no items to be selected or editted
				this.CustomPropertyDataGrid.SelectedIndex = -1;
				this.CustomPropertyDataGrid.EditItemIndex = -1;
			}
		}
		
		
		/// <summary>
		/// Event fired when an item is bound to the datagrid.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void CustomPropertyDataGrid_DoItemDataBound(object sender, System.Web.UI.WebControls.DataGridItemEventArgs e)
		{
			if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
			{
				LinkButton b = (LinkButton)e.Item.FindControl("DeleteButton");
				b.Attributes.Add("onclick", "return confirm('Are you sure you want to delete this property?');");
			}
		}


		/// <summary>
		/// Handles all events for the <see cref="DataGrid"/> (edit, delete, update).
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		private void CustomPropertyDataGrid_DoItemCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			switch (e.CommandName.ToUpper())
			{
				case "ADD":
					this.CreateProperty(this.GetGuid(), 
						((TextBox)e.Item.FindControl("NewPropertyKeyTextBox")).Text.ToString(),
						((TextBox)e.Item.FindControl("NewPropertyValueTextBox")).Text.ToString());
					((TextBox)e.Item.FindControl("NewPropertyKeyTextBox")).Text = string.Empty;
					((TextBox)e.Item.FindControl("NewPropertyValueTextBox")).Text = string.Empty;
					break;
				case "EDIT":
					this.CustomPropertyDataGrid.EditItemIndex = e.Item.ItemIndex;
					break;
				case "UPDATE":
					this.UpdateProperty(e.CommandArgument.ToString(), 
						((TextBox)e.Item.FindControl("PropertyKeyTextBox")).Text.ToString(),
						((TextBox)e.Item.FindControl("PropertyValueTextBox")).Text.ToString());
					this.CustomPropertyDataGrid.EditItemIndex = -1;
					break;
				case "CANCEL":
					this.CustomPropertyDataGrid.SelectedIndex = -1;
					this.CustomPropertyDataGrid.EditItemIndex = -1;
					break;
				case "DELETE":
					this.DeleteProperty(e.CommandArgument.ToString());
					break;
			}

			this.BindProperties();
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.CustomPropertyDataGrid.ItemCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.CustomPropertyDataGrid_DoItemCommand);
			this.CustomPropertyDataGrid.ItemDataBound += new System.Web.UI.WebControls.DataGridItemEventHandler(this.CustomPropertyDataGrid_DoItemDataBound);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}