#region Namespace references
/// Microsoft .NET Framework namespaces
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Summary description for TemplateManager.
	/// </summary>
	public class TemplateManager : System.Web.UI.Page
	{
		#region Constants
		#endregion

		#region Objects
		private CustomPropertyTemplate m_templates = new CustomPropertyTemplate();
		#endregion
		
		#region Designer Objects
		protected System.Web.UI.WebControls.DropDownList TemplateDropDownList;
		protected System.Web.UI.WebControls.LinkButton NewTemplateLinkButton;
		protected System.Web.UI.WebControls.LinkButton DeleteTemplateLinkButton;
		protected System.Web.UI.WebControls.LinkButton RenameTemplateLinkButton;
		protected System.Web.UI.WebControls.TextBox TemplateNamePlaceholderTextBox;
		protected System.Web.UI.WebControls.DataGrid TemplatePropertyGrid;
		#endregion

		#region Constructor & Descructor
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Load the template dataset if it isn't already loaded.
		/// </summary>
		private void BindTemplateSelector()
		{
			if (this.m_templates != null)
			{
				DataRow[] dr = this.m_templates.Templates.Tables[0].Select(string.Empty, "Name ASC");
				this.TemplateDropDownList.DataSource = dr;
				this.TemplateDropDownList.DataValueField = "Guid";
				this.TemplateDropDownList.DataTextField = "Name";
				this.TemplateDropDownList.DataBind();
			}
		}


		/// <summary>
		/// Bind the selected template properties to the grid.
		/// </summary>
		private void BindTemplateProperties(CpDbExTemplate.PropertyRow[] properties)
		{
			this.TemplatePropertyGrid.DataSource = properties;
			this.TemplatePropertyGrid.DataBind();
		}


		/// <summary>
		/// Adds a property to the template.
		/// </summary>
		/// <param name="template">Template to add the property to.</param>
		/// <param name="propertyKey">Key value of the new property.</param>
		/// <param name="propertyValue">Value of the new property.</param>
		private void AddProperty(CpDbExTemplate.TemplateRow template, string propertyKey, string propertyValue)
		{
			// create a new property record
			CpDbExTemplate.PropertyRow property = this.m_templates.Templates.Property.NewPropertyRow();

			// set the values
			property.TemplateRow = template;
			property.Guid = Guid.NewGuid().ToString();
			property.Key = propertyKey;
			property.Value = propertyValue;

			// add it to the dataset & save the changes
			this.m_templates.Templates.Property.AddPropertyRow(property);
			this.m_templates.SaveChanges();

			// reload the changes
			this.m_templates = new CustomPropertyTemplate();
		}


		/// <summary>
		/// Updates the current property value.
		/// </summary>
		/// <param name="propertyGuid">Unique Id of the property.</param>
		/// <param name="propertyKey">Key value of the property.</param>
		/// <param name="propertyValue">Value of the property.</param>
		private void UpdateProperty(string propertyGuid, string propertyKey, string propertyValue)
		{
			CpDbExTemplate.PropertyRow[] property = (CpDbExTemplate.PropertyRow[])this.m_templates.Templates.Property.Select("Guid='" +propertyGuid +"'");

			property[0].Key = propertyKey;
			property[0].Value = propertyValue;
		
			// save the changes
			this.m_templates.SaveChanges();

			// reload the changes
			this.m_templates = new CustomPropertyTemplate();
		}


		/// <summary>
		/// Deletes the specified property from the collection.
		/// </summary>
		/// <param name="propertyGuid">Unique ID of the property.</param>
		private void DeleteProperty(string propertyGuid)
		{
			CpDbExTemplate.PropertyRow[] property = (CpDbExTemplate.PropertyRow[])this.m_templates.Templates.Property.Select("Guid='" +propertyGuid +"'");
			this.m_templates.Templates.Property.RemovePropertyRow(property[0]);
				
			// save the changes
			this.m_templates.SaveChanges();

			// reload the changes
			this.m_templates = new CustomPropertyTemplate();
		}


		/// <summary>
		/// Creates a new template & reloads the selector and property list.
		/// </summary>
		private void CreateNewTemplate()
		{
			// get the new template name (make sure a name was specified
			if (this.TemplateNamePlaceholderTextBox.Text.ToString().Trim() == string.Empty)
			{
				this.Page.RegisterClientScriptBlock("NoNameError","<script language=javascript>alert(\"You must specify a name for the template before it will be created.\");</script>");
				return;
			}

			// get the new template name
			string newTemplateName = this.TemplateNamePlaceholderTextBox.Text.ToString();

			// create & save it
			CpDbExTemplate.TemplateRow template =  this.m_templates.Templates.Template.NewTemplateRow();
			template.Guid = Guid.NewGuid().ToString();
			template.Name = newTemplateName;
			this.m_templates.Templates.Template.AddTemplateRow(template);
			this.m_templates.SaveChanges();

			// rebind the drop down list
			this.BindTemplateSelector();
			// determine which item in the drop down list is the new one, and select it
			foreach (ListItem li in this.TemplateDropDownList.Items)
				if (li.Value == template.Guid)
				{
					li.Selected = true;
					break;
				}
			// rebind the empty property list
			this.TemplateDropDownList_DoSelectedIndexChanged(null, null);
		}


		/// <summary>
		/// Renames the selected template.
		/// </summary>
		private void RenameTemplate()
		{
			// get the new template name (make sure a name was specified
			if (this.TemplateNamePlaceholderTextBox.Text.ToString().Trim() == string.Empty)
			{
				this.Page.RegisterClientScriptBlock("NoNameError","<script language=javascript>alert(\"You must specify a name for the template before the selected template can be renamed.\");</script>");
				return;
			}

			// get the new template name
			string newTemplateName = this.TemplateNamePlaceholderTextBox.Text.ToString();

			// get a reference to the curretly selected template & rename it.
			CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])this.m_templates.Templates.Template.Select("Guid = '" +this.TemplateDropDownList.SelectedValue.ToString() +"'");
			template[0].Name = newTemplateName;
			this.m_templates.SaveChanges();

			// rebind the drop down list
			this.BindTemplateSelector();
			// determine which item in the drop down list is the new one, and select it
			foreach (ListItem li in this.TemplateDropDownList.Items)
				if (li.Value == template[0].Guid.ToString())
				{
					li.Selected = true;
					break;
				}
		}


		/// <summary>
		/// Deletes the selected template.
		/// </summary>
		private void DeleteTemplate()
		{
			// get a reference to the curretly selected template & rename it.
			CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])this.m_templates.Templates.Template.Select("Guid = '" +this.TemplateDropDownList.SelectedValue.ToString() +"'");
			this.m_templates.Templates.Template.RemoveTemplateRow(template[0]);
			this.m_templates.SaveChanges();

			// rebind the drop down list
			this.BindTemplateSelector();
			// select the first template in the list
			this.TemplateDropDownList.Items[0].Selected = true;
			this.TemplateDropDownList_DoSelectedIndexChanged(null, null);
		}
		#endregion

		#region Public methods
		#endregion

		#region Public properties
		#endregion

		#region Event Handlers
		/// <summary>
		/// Hanles event when the page is loaded.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!IsPostBack)
			{
				// bind the template dropdown list
				this.BindTemplateSelector();
				
				// bind the first template in the list
				CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])this.m_templates.Templates.Template.Select(string.Empty, "Name ASC");
				this.BindTemplateProperties(template[0].GetPropertyRows());

				// set the two linkbutton onclick events
				this.NewTemplateLinkButton.Attributes.Add("onclick","SetTemplateName();");
				this.RenameTemplateLinkButton.Attributes.Add("onclick","SetTemplateName();");
				this.DeleteTemplateLinkButton.Attributes.Add("onclick","return confirm(\"Are you sure you want to delete this template?  This is not reverseable.\");");
			}
		}


		/// <summary>
		/// Handles all events for the <see cref="DataGrid"/> (edit, delete, update).
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		private void TemplatePropertyGrid_DoItemCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			// get current template
			CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])this.m_templates.Templates.Template.Select("Guid = '" +this.TemplateDropDownList.SelectedValue.ToString() +"'");

			switch (e.CommandName.ToString().ToUpper())
			{
				case "ADD":
					this.AddProperty(template[0],
						((TextBox)e.Item.FindControl("NewPropertyKeyTextBox")).Text.ToString(),
						((TextBox)e.Item.FindControl("NewPropertyValueTextBox")).Text.ToString());
					break;
				case "CANCEL":
					this.TemplatePropertyGrid.SelectedIndex = -1;
					this.TemplatePropertyGrid.EditItemIndex = -1;
					break;
				case "EDIT":
					this.TemplatePropertyGrid.EditItemIndex = e.Item.ItemIndex;
					break;
				case "DELETE":
					this.DeleteProperty(e.CommandArgument.ToString());
					break;
				case "UPDATE":
					this.UpdateProperty(e.CommandArgument.ToString(),
						((TextBox)e.Item.FindControl("PropertyKeyTextBox")).Text.ToString(),
						((TextBox)e.Item.FindControl("PropertyValueTextBox")).Text.ToString());
					break;
			}

			// rebind the properties
			this.BindTemplateProperties(template[0].GetPropertyRows());
		}


		/// <summary>
		/// Event fired when an item is bound to the datagrid.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void TemplatePropertyGrid_DoItemDataBound(object sender, System.Web.UI.WebControls.DataGridItemEventArgs e)
		{
			if (e.Item.ItemType == ListItemType.Item || e.Item.ItemType == ListItemType.AlternatingItem)
			{
				LinkButton b = (LinkButton)e.Item.FindControl("DeleteLinkButton");
				b.Attributes.Add("onclick", "return confirm('Are you sure you want to delete this property?');");
			}
		}


		/// <summary>
		/// Handles event fired when the selected index changes in the dropdownlist.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void TemplateDropDownList_DoSelectedIndexChanged(object sender, System.EventArgs e)
		{
			// get the template properties
			CpDbExTemplate.TemplateRow[] template = (CpDbExTemplate.TemplateRow[])this.m_templates.Templates.Template.Select("Guid = '" +this.TemplateDropDownList.SelectedValue.ToString() +"'");

			// bind the properties
			this.BindTemplateProperties(template[0].GetPropertyRows());
		}

	
		/// <summary>
		/// Handles click event for new template icon.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void NewTemplateLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.CreateNewTemplate();
		}


		/// <summary>
		/// Handles click event for the deletion of the currently selected template.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void DeleteTemplateLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.DeleteTemplate();
		}


		/// <summary>
		/// Handles click event for the renaming of the currently selected template.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RenameTemplateLinkButton_DoClick(object sender, System.EventArgs e)
		{
			this.RenameTemplate();
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.TemplateDropDownList.SelectedIndexChanged += new System.EventHandler(this.TemplateDropDownList_DoSelectedIndexChanged);
			this.NewTemplateLinkButton.Click += new System.EventHandler(this.NewTemplateLinkButton_DoClick);
			this.DeleteTemplateLinkButton.Click += new System.EventHandler(this.DeleteTemplateLinkButton_DoClick);
			this.RenameTemplateLinkButton.Click += new System.EventHandler(this.RenameTemplateLinkButton_DoClick);
			this.TemplatePropertyGrid.ItemCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.TemplatePropertyGrid_DoItemCommand);
			this.TemplatePropertyGrid.ItemDataBound += new System.Web.UI.WebControls.DataGridItemEventHandler(this.TemplatePropertyGrid_DoItemDataBound);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}