#region References
using System;
#endregion

namespace ContentManagement.Extensions
{
	/// <summary>
	/// Summary description for DataHelper.
	/// </summary>
	internal class DataHelper
	{
		#region Boilerplate
		#region Constructor
		private DataHelper(){}
		#endregion
		#endregion

		#region Exposed class methods
		/// <summary>
		/// Returns null if an object is null
		/// </summary>
		internal static string CheckNullString(object obj)
		{
			if(obj is DBNull)
				return null;
		
			return (string)obj;
		}


		/// <summary>
		/// Returns null if a string is empty
		/// </summary>
		internal static object CheckNull(string text)
		{
			if(text != null && text.Trim().Length > 0)
				return text;
		
			return DBNull.Value;
		}


		/// <summary>
		/// Returns null if the object is null
		/// </summary>
		internal static string CheckNull(object obj)
		{
			return (string) obj;
		}


		/// <summary>
		/// Returns null if DBNull is provided.
		/// </summary>
		internal static string CheckNull(DBNull obj)
		{
			return null;
		}


		/// <summary>
		/// Returns null if a number is not a min. number.
		/// </summary>
		internal static object CheckNullInt(int num)
		{
			if (num >=0)
				return num;
	
			return DBNull.Value;
		}


		/// <summary>
		/// Returns null if a date is not existant
		/// </summary>
		internal static object CheckNullDateTime(object dt)
		{
			DateTime tmp;

			try
			{tmp = Convert.ToDateTime(dt);}
			catch
			{return DBNull.Value;}

			if (tmp.Year < 1755)
				return DBNull.Value;
			return tmp;
		}


		/// <summary>
		/// Returns -1 if the object's value is DBNull
		/// </summary>
		internal static int CheckNullDBInt(object obj)
		{
			if (obj is System.DBNull)
				return -1;
			return Convert.ToInt32(obj);
		}
		#endregion
	}
}
