#region Namespace references
using System;
using System.ComponentModel;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
#endregion

namespace IIS_CmsExtensionAdmin
{
	/// <summary>
	/// Message panel object.
	/// </summary>
	[ToolboxData("<{0}:MessagePanel runat=\"server\"></{0}:MessagePanel>")]
	public class MessagePanel : Panel, INamingContainer
	{
		#region Constants
		private const string c_vskey_message = "Message";
		private const string c_vskey_error = "Error";
		private const string c_vskey_warning = "Warning";

		private const string c_iconCritical = "~/images/ico_critical.gif";
		private const string c_iconInfo = "~/images/ico_info.gif";
		private const string c_iconWarning = "~/images/ico_warning.gif";
		#endregion

		#region Objects
		bool m_showMessagePanel;
		bool m_showErrorPanel;
		bool m_showWarningPanel;

		string m_messageCssClass;
		string m_errorCssClass;
		string m_warningCssClass;
		string m_messageIconUrl;
		string m_warningIconUrl;
		string m_errorIconUrl;
		#endregion

		#region Constructor & destructor
		public MessagePanel()
		{
			ViewState[c_vskey_message] = String.Empty;
			ViewState[c_vskey_error] = String.Empty;
			ViewState[c_iconWarning] = String.Empty;
		}
		#endregion

		#region Public properties
		public string Message
		{
			get { return (string)ViewState[c_vskey_message]; }
			set { ViewState[c_vskey_message] = value; }
		}

		public string Error
		{
			get { return (string)ViewState[c_vskey_error]; }
			set { ViewState[c_vskey_error] = value; }
		}

		public string Warning
		{
			get { return (string)ViewState[c_vskey_warning];}
			set { ViewState[c_vskey_warning] = value;}
		}

		public bool ShowMessagePanel
		{
			get { return m_showMessagePanel; }
			set { m_showMessagePanel = value; }
		}

		public bool ShowErrorPanel
		{
			get { return m_showErrorPanel; }
			set { m_showErrorPanel = value; }
		}

		public bool ShowWarningPanel
		{
			get { return m_showWarningPanel; }
			set { m_showWarningPanel = value; }
		}

		public string MessageCssClass
		{
			get 
			{
				if(this.m_messageCssClass == null)
				{
					this.m_messageCssClass = "MessagePanel";
				}
				return m_messageCssClass; 
			}
			set { m_messageCssClass = value; }
		}

		public string WarningCssClass
		{
			get 
			{
				if(this.m_warningCssClass == null)
				{
					this.m_warningCssClass = "MessagePanel";
				}
				return m_warningCssClass; 
			}
			set { m_warningCssClass = value; }
		}

		public string ErrorCssClass
		{
			get 
			{
				if(this.m_errorCssClass == null)
				{
					this.m_errorCssClass = "ErrorPanel";
				}
				return m_errorCssClass; 
			}
			set { m_errorCssClass = value; }
		}

		public string MessageIconUrl
		{
			get 
			{
				if(m_messageIconUrl == null)
				{
					m_messageIconUrl = c_iconInfo;
				}
				return m_messageIconUrl;
			}
			set {m_messageIconUrl = value; }
		}

		public string WarningIconUrl
		{
			get 
			{
				if(m_warningIconUrl == null)
				{
					m_warningIconUrl = c_iconWarning;
				}
				return m_warningIconUrl;
			}
			set { m_warningIconUrl = value; }
		}

		public string ErrorIconUrl
		{
			get 
			{
				if(m_errorIconUrl == null)
				{
					m_errorIconUrl = c_iconCritical;
				}
				return m_errorIconUrl;
			}
			set { m_errorIconUrl = value; }
		}

		#endregion

		#region Event handlers
		/// <summary>
		/// Overrides the base OnInit method
		/// </summary>
		/// <param name="e"></param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);
		}


		/// <summary>
		/// Overrides base render object.  Writes out the panel
		/// </summary>
		/// <param name="writer"><see cref="HtmlTextWriter">HtmlTextWriter</see></param>
		protected override void Render(HtmlTextWriter writer)
		{
			if (null != Page) 
				Page.VerifyRenderingInServerForm(this);

			if (this.ShowErrorPanel)
			{			
				Panel errorPanel = BuildPanel(this.Error, this.ErrorCssClass, this.ErrorIconUrl);
				this.Controls.Add(errorPanel);
			}

			if (this.ShowMessagePanel)
			{			
				Panel messagePanel = BuildPanel(this.Message, this.MessageCssClass, this.MessageIconUrl);
				this.Controls.Add(messagePanel);
			}

			if (this.ShowWarningPanel)
			{			
				Panel warningPanel = BuildPanel(this.Warning, this.WarningCssClass, this.WarningIconUrl);
				this.Controls.Add(warningPanel);
			}

			base.Render(writer);
		}
		#endregion

		#region Private/protected/internal methods
		/// <summary>
		/// Creates an ASP.NET panel message block.
		/// </summary>
		/// <param name="messageText">Message to display.</param>
		/// <param name="cssClass">CSS class to use in formatting.</param>
		/// <param name="imageUrl">URL of icon image.</param>
		/// <returns>ASP.NET panel.</returns>
		protected virtual Panel BuildPanel(string messageText, string cssClass, string imageUrl)
		{
			Panel result = new Panel();
							
			if (null != imageUrl && cssClass.Length > 0) result.CssClass = cssClass;
	
			if (null != imageUrl && imageUrl.Length > 0) 
			{
				System.Web.UI.WebControls.Image image = new System.Web.UI.WebControls.Image();
				image.ImageUrl = imageUrl;
				//image.Attributes.Add("src", imageUrl);
				result.Controls.Add(image);
			}

			Panel message = new Panel();
			message.Controls.Add(new LiteralControl(messageText));
			result.Controls.Add(message);

			return result;
		}
		#endregion

		#region Public methods
		/// <summary>
		/// Creates message object.
		/// </summary>
		/// <param name="message">Message to display.</param>
		public void ShowMessage(string message)
		{
			ShowMessage(message, true);
		}


		/// <summary>
		/// Creates message object.
		/// </summary>
		/// <param name="message">Message to display.</param>
		/// <param name="clearExistingMessages">Flag indicating if previous messages should be cleared.</param>
		public void ShowMessage(string message, bool clearExistingMessages)
		{
			if (clearExistingMessages) 
				this.Message = message;
			else
				this.Message += " " + message;

			this.ShowMessagePanel = true;
			this.Visible = true;
		}


		/// <summary>
		/// Creates warning object.
		/// </summary>
		/// <param name="warning">Warning to display.</param>
		public void ShowWarning(string warning)
		{
			ShowWarning(warning, true);
		}


		/// <summary>
		/// Creates warning object.
		/// </summary>
		/// <param name="warning">warning to display.</param>
		/// <param name="clearExistingMessages">Flag indicating if previous messages should be cleared.</param>
		public void ShowWarning(string warning, bool clearExistingMessages)
		{
			if (clearExistingMessages) 
				this.Warning = warning;
			else
				this.Warning += " " + warning;

			this.ShowWarningPanel = true;
			this.Visible = true;
		}


		/// <summary>
		/// Creates error message
		/// </summary>
		/// <param name="message">Message to display.</param>
		public void ShowError(string message)
		{
			ShowError(message, true);
		}

	
		/// <summary>
		/// Creates error message
		/// </summary>
		/// <param name="message">Message to display.</param>
		/// <param name="clearExistingMessages">Flag indicating if previous messages should be cleared.</param>
		public void ShowError(string message, bool clearExistingMessages)
		{
			if (clearExistingMessages) 
				this.Error = message;
			else
				this.Error += " " + message;

			this.ShowErrorPanel = true;
			this.Visible = true;
		}


		/// <summary>
		/// Hides the object.
		/// </summary>
		public void Clear()
		{
			this.Visible = false;
		}
		#endregion
	}
}